#include <iostream>
#include <string.h>
#include "Config.h"

using namespace std;

PlayerConfig::PlayerConfig()
{
#ifdef WIN32
	strcpy_s(team,5, "Test");
#else
	strcpy(team, "Test");
#endif
	id = 1;
}


Config::Config()
{
#ifdef WIN32
    strcpy_s(serverIP,100, "localhost");
#else
	strcpy(serverIP, "localhost");
#endif
	serverPort = 3100;
    connectionType = CONN_TCP;

	player = new PlayerConfig();

	server.sin_family = AF_INET;
}

Config::~Config()
{
	delete player;
}

bool Config::getAttributeString(TiXmlElement * pElement, const char *pAttribute, char *string, unsigned int stringSize)
{
    if (!pElement)
        return 0;
    
    TiXmlAttribute *pAttrib = pElement->FirstAttribute();
    
    while (pAttrib)
	{
        if (!strcasecmp(pAttrib->Name(), pAttribute))
		{
#ifdef WIN32
			strncpy_s(string, stringSize - 1,pAttrib->Value(),(int)strlen(pAttrib->Value()));
#else
			strncpy(string, pAttrib->Value(), stringSize - 1);            
#endif
            string[stringSize - 1] = '\0';
            return true;
        }
        pAttrib = pAttrib->Next();
    }
    
    return false;
}

bool Config::getAttributeInt(TiXmlElement * pElement, const char *pAttribute, int &integer)
{
    if (!pElement)
        return 0;
    
    TiXmlAttribute *pAttrib = pElement->FirstAttribute();
    
    while (pAttrib)
	{
        if (!strcasecmp(pAttrib->Name(), pAttribute))
		{
            if (pAttrib->QueryIntValue(&integer) == TIXML_SUCCESS)
                return true;
        }
        
        pAttrib = pAttrib->Next();
    }
    
    return false;
}

bool Config::getAttributeDouble(TiXmlElement * pElement, const char *pAttribute, double &num)
{
    if (!pElement)
        return 0;
    
    TiXmlAttribute *pAttrib = pElement->FirstAttribute();
    
    while (pAttrib)
	{
        if (!strcasecmp(pAttrib->Name(), pAttribute))
		{
            if (pAttrib->QueryDoubleValue(&num) == TIXML_SUCCESS)
                return true;
        }
        
        pAttrib = pAttrib->Next();
    }
    
    return false;
}

int Config::loadConfig(const char *xmlFile)
{
    if (!xmlDocument.LoadFile(xmlFile))
	{
        cerr << "Failed to open config file " << xmlFile << endl;
        return 1;
    }
    
    TiXmlNode *xmlNode = xmlDocument.FirstChild("Sirius");
    
    if (xmlNode)
	{
        TiXmlNode *connNode = xmlNode->FirstChild("Connection");
        
        if (connNode) 
		{
            getAttributeString(connNode->ToElement(), "IP", serverIP, sizeof(serverIP));
            
#ifdef WIN32
			unsigned long addr = inet_addr(serverIP);
			if (addr != INADDR_ANY && addr != INADDR_NONE)
#else
			in_addr_t addr = inet_addr(serverIP);
			if (addr != (in_addr_t) -1)
#endif
			{
				server.sin_addr.s_addr = addr;
			}
			else
			{
				cout <<"Chybny format IP adresy" << endl;
			}

            getAttributeInt(connNode->ToElement(), "Port", (int &) serverPort);

			if (serverPort > 0 && serverPort < 65536)
			{
				server.sin_port = htons(serverPort);
			}
			else
			{
				cout <<"Cislo portu mimo rozsah (0,65526)" << endl;
				exit(0);
			}

			memset(&server.sin_zero, 0, sizeof(server.sin_zero));
        }

		char connType[10];
		getAttributeString(connNode->ToElement(), "Type", connType, sizeof(connType));
		if(strcmp(connType,"tcp") == 0)
		{
			connectionType = CONN_TCP;
		}

		else if(strcmp(connType,"udp") == 0)
		{
			connectionType = CONN_UDP;
		}

		else
		{
			cout <<"Neznamy typ protokolu. Pouzije sa TCP";
		}


        TiXmlNode *playerNode = xmlNode->FirstChild("Player");
        if (playerNode)
		{
            getAttributeString(playerNode->ToElement(), "Team", player->team, sizeof(player->team));
            getAttributeInt(playerNode->ToElement(), "Number", player->id);
        }
        
    }
    return 0;
}
