/*
 * main.cpp
 */

#include "main/Agent.h"
#include <iostream>
#include <signal.h>

using namespace std;

#define XML_CONFIG_FILE "ourteam.xml"

Config *config;
char *config_file = XML_CONFIG_FILE;

bool run = true;

void signalINT(int signal)
{
	run = false;
}

void printCommandLineHelp()
{
	cout << "Sirius - Agent player for RoboCup3D by Hviezdna jedenastka (2008), Marian Buchta, Michal Kvetan" << endl << endl;
	cout << "usage: Sirius [arguments]" << endl << endl;
	cout << "Arguments:" << endl;
	cout << "  -h, --help\t\t\tPrint help (this message) and exit." << endl;
	cout << "  -c, --configfile FILE\t\tConfig file, default 'agent.xml'." << endl;
	cout << "  -skill 1\t\tSkill: moving hands" << endl;
	cout << "  -skill 2\t\tSkill: turning around" << endl;
	cout << "  -skill 3\t\tSkill: walking" << endl;
	cout << "  -skill 4\t\tSkill: saluting" << endl;
	cout << "  -skill 5\t\tSkill: walking 2" << endl;
	cout << "  -skill 6\t\tSkill: doing squat" << endl;
	cout << "  -skill 7\t\tSkill: move left" << endl;
}

int parseCommandLineArguments(int argc, char *const argv[])
{
	int returnSkill = 0;
	for (int i = 1; i < argc; i++)
	{

		if (strcmp(argv[i], "-h") == 0 || strcmp(argv[i], "--help") == 0)
		{
			printCommandLineHelp();
			exit(0);
		}

		if (strcmp(argv[i], "-c") == 0 || strcmp(argv[i], "--configfile") == 0)
		{
			if (i + 1 < argc)
			{
				config_file = argv[i + 1];
			}
		}

		if (strcmp(argv[i], "-skill") == 0 )
		{
			if (i + 1 < argc)
			{
				returnSkill = atoi(argv[i + 1]);
			}
		}
	}
	return returnSkill;
}


int main(int argc, char *const argv[])
{
	signal(SIGINT, signalINT);
	
	int skill = parseCommandLineArguments(argc, argv);
	
	config = new Config();
	
	if (config->loadConfig(config_file)) 
	{
		return 1;
	}

	Agent * agent = Agent::getInstance();
	agent->initialize(&run, config->connectionType, (struct sockaddr *)&config->server, config->player->team,&config->player->id,skill);
	agent->start();

	delete config;
	return 0;
}
