/*
 * Communication.cpp
 */
#include "Communication.h"
#include <string.h>

Communication::Communication(ConnectionType protocol)
{
	tcpSocket = NULL;
	udpSocket = NULL;

	if (protocol == CONN_TCP)
	{
		tcpSocket = new TCPSocket();
	}
	else if (protocol == CONN_UDP)
	{
		udpSocket = new UDPSocket();
	}
	server = NULL;
}

bool Communication::setServer(struct sockaddr *server)
{
	if (tcpSocket != NULL)
	{
		return tcpSocket->connect(server);
	}
	else if (udpSocket != NULL)
	{
		this->server = server;
		return true;
	}
	else
	{
		return false;
	}
}

bool Communication::send(const char *message)
{
	if (tcpSocket != NULL)
	{
		return tcpSocket->send(message);
	}
	else if (udpSocket != NULL)
	{
		return udpSocket->sendto(message, server);
	}
	else
	{
		return false;
	}
}

bool Communication::select(int seconds, int microseconds)
{
	if (tcpSocket != NULL)
	{
		return tcpSocket->select(seconds, microseconds);
	}
	else if (udpSocket != NULL)
	{
		return udpSocket->select(seconds, microseconds);
	}
	else
	{
		return false;
	}
}

bool Communication::receive(char *message, int messageSize)
{
	if (tcpSocket != NULL)
	{
		return tcpSocket->receive(message, messageSize);
	}
	else if (udpSocket != NULL)
	{
		return udpSocket->receivefrom(message, messageSize, server);
	}
	else
	{
		return false;
	}
}

Communication::~Communication()
{
	if (tcpSocket != NULL)
	{
		delete tcpSocket;
	}
	else if (udpSocket != NULL)
	{
		delete udpSocket;
	}
}
