#include "AgentModel.h"

AgentModel * AgentModel::INSTANCE = new AgentModel();

AgentModel * AgentModel::getInstance() {
	return INSTANCE;
}

AgentModel::AgentModel()
{
	mJointIDMap =  SetupJointIDMap(mJointIDMap);	
	gyroscope = new Gyroscope();
	FRP = new ForceResistance();
	initSkillMoveHands();
	initSkillRound();
	initSkillMove();
}

string AgentModel::Action()
{
	return generate();
}

void AgentModel::update(ServerMsgParserData *parsedData)
{
	simTime = parsedData->GetSimTime();
	gyroscope->update(parsedData->GetGyroRate(), simTime);

	FRP->update(parsedData->GetFrpLocCoord_LF(), parsedData->GetFrpLocCoord_RF(),
		parsedData->GetFrpForceVector_LF(), parsedData->GetFrpForceVector_RF());
	
	mHingeJointSenseMap = parsedData->GetmHingeJointSenseMap();
}

//tvorba akcii
string AgentModel::generate()
{
	if (skill == 1) {
		return executeSkillMoveHands();
	};
	if (skill == 2) {
		return executeSkillRound();
	};
	if (skill == 3) {
		return executeSkillMove();
	};
	if (skill == 4) {
		return executeSkillSalute();
	};
	if (skill == 5) {
		return executeSkillMove2();
	};
	if (skill == 6) {
		return executeSquat();
	};
	if (skill == 7) {
		return executeMoveLeft();
	}
	if (skill == 8) {
		return executeSkillRun();
	};
	if (skill == 9) {
		return executeSkillDrepuj();
	}
	if (skill == 10) {
		return executeFallDown();
	}
	return "";
}

AgentModel::~AgentModel()
{
	delete gyroscope;
	delete FRP;
}


std::string gJointEff_2_Str[] = 
{
	"he1",
	"he2",
	"lle1",
	"rle1",
	"lle2",
	"rle2",
	"lle3",
	"rle3",
	"lle4",
	"rle4",
	"lle5",
	"rle5",
	"lle6",
	"rle6",
	"lae1",
	"rae1",
	"lae2",
	"rae2",
	"lae3",
	"rae3",
	"lae4",
	"rae4",
};

double AgentModel::GetJointAngleDeg(JointID jid)
{
	return mHingeJointSenseMap[jid].angle;
}


double AgentModel::GetJointAngleRad(JointID jid)
{
	return GetJointAngleDeg(jid) * PI / 180.0;
}

void AgentModel::initSkillMoveHands() {
	type_1 = 0;
	step_1 = 0;
}

string AgentModel::executeSkillMoveHands() {
	ss.str("");
	cout << "hi "<<type_1<<"  "<<GetJointAngleDeg(JID_LARM_2)<<"  "<<GetJointAngleDeg(JID_RARM_2) <<"\n" ;
	if (type_1 == 0) {
		
		if (GetJointAngleDeg(JID_LARM_2) < 90.0) {
			ss << "(lae2 1.0)";
		}

		if (GetJointAngleDeg(JID_RARM_2) > -90.0) {
			ss << "(rae2 -1.0)";
		}

		if (GetJointAngleDeg(JID_LARM_2) >= 90.0 && GetJointAngleDeg(JID_RARM_2) <= -90.0) {
			type_1 = 1;
		}
	} else {
		if (GetJointAngleDeg(JID_LARM_2) > 0.0) {
			ss << "(lae2 -1.0)";
		}
		if (GetJointAngleDeg(JID_RARM_2) < 0.0) {
			ss << "(rae2 1.0)";
		}

		if (GetJointAngleDeg(JID_LARM_2) <= 0.0 && GetJointAngleDeg(JID_RARM_2) >= 0.0) {
			type_1 = 0;
		}
	}
	
	return ss.str();
}

void AgentModel::initSkillRound() 
{
	faza = 1;
	counter = 0;
}
string AgentModel::executeSkillRound()
{
	ss.str("");
	cout <<GetJointAngleDeg(JID_LLEG_3) << "  "<<GetJointAngleDeg(JID_LLEG_4)<<"  "<<GetJointAngleDeg(JID_LLEG_6)<<"\n";
	//cout <<GetJointAngleDeg(JID_LLEG_1) << "  "<<GetJointAngleDeg(JID_RLEG_1)<<"\n";
	if (counter < 10) {
	   if (faza == 1) 
		{	
			if (GetJointAngleDeg(JID_RLEG_6) > -10.0) {
				ss << "(rle6 -0.5)";
			} else {
				ss << "(rle6 0.0)";
				faza = 2;
			}
		}
		if (faza == 2) 
		{
			if (GetJointAngleDeg(JID_LLEG_3) < 25.0) {
				ss << "(lle3 0.5)";
			} else {
				ss << "(lle3 0.0)";
				faza =3;
			}
		}
		if (faza == 3) 
		{
			if (GetJointAngleDeg(JID_LLEG_6) > -5.0) {
				ss << "(lle6 -0.5)";
			} else {
			  	ss << "(lle6 0.0)";
				faza =4;
			}
		}
		if (faza == 4) {
			if (GetJointAngleDeg(JID_LLEG_3) > 5.0) 
			{
				ss << "(lle3 -0.5)";
			} else {
				ss << "(lle3 0.0)";
				faza =2;
				counter ++;
			}
		}
	}
	
	
	return ss.str();
}

void AgentModel::initSkillMove() 
{
	step = 0;
	foot = 1 ;
}

string AgentModel::executeSkillMove() 
{
	ss.str("");
	cout <<GetJointAngleDeg(JID_LLEG_3) << "  "<<GetJointAngleDeg(JID_LLEG_4)<<"  "<<GetJointAngleDeg(JID_LLEG_6)<<"\n";
	if(step == 0)
	{
		if(GetJointAngleDeg(JID_LARM_1) > -85.0)
		{
			ss << "(lae1 -1.0)(rae1 -1.0)";
		}
		else
		{
			ss << "(lae1 0.0)(rae1 0.0)";
			step=1;
		}
	}

	if (foot == 1)
	{
		if (step == 1) 
		{	
			if (GetJointAngleDeg(JID_LLEG_3) < 25.0) {
				ss << "(lle3 1.0)(lae1 3.0)";
			} else {
				ss << "(lle3 0.0)(lae1 0.0)";
				step = 2;
			}
		}
		if (step == 2) 
		{
			if (GetJointAngleDeg(JID_LLEG_3) > 10.0) {
				ss << "(lle3 -1.0)(lae1 -3.0)";
			} else {
				ss << "(lle3 0.0)(lae1 0.0)";
				step = 1;
				foot = 2;
			}
		}
	} else {
		if (step == 1) 
		{	
			if (GetJointAngleDeg(JID_RLEG_3) < 25.0) {
				ss << "(rle3 1.0)(rae1 3.0)";
			} else {
				ss << "(rle3 0.0)(rae1 0.0)";
				step = 2;
			}
		}
		if (step == 2) 
		{
			if (GetJointAngleDeg(JID_RLEG_3) > 10.0) {
				ss << "(rle3 -1.0)(rae1 -3.0)";
			} else {
				ss << "(rle3 0.0)(rae1 0.0)";
				step = 1;
				foot = 1;
			}
		}
	}
	return ss.str();
}
string AgentModel::executeSkillSalute()
{
	ss.str("");
	cout << "Phase: " << faza << " " << GetJointAngleDeg(JID_RARM_1) << "  "<<GetJointAngleDeg(JID_RARM_2)<<"\n";	
	if(GetJointAngleDeg(JID_LARM_1) > -90.0)
	{
		ss << "(lae1 -1.0)";
	}
	else
	{
		ss << "(lae1 0.0)";
	}
	if(faza == 1)
	{
		if(GetJointAngleDeg(JID_RARM_1) < 50.0)
		{
			ss << "(rae1 1.0)";
		}
		else
		{
			faza = 2;
		}
	}
	else if(faza == 2)
	{
		if(GetJointAngleDeg(JID_RARM_2) > -40)
		{
			ss << "(rae1 -0.5)(rae2 -1.0)";
		}
		else
		{
			faza = 3;
		}
	}
	else if(faza == 3)
	{
		if(GetJointAngleDeg(JID_RARM_2) < -10)
		{
			ss << "(rae1 0.5)(rae2 1.0)";
		}
		else
		{
			faza = 2;
		}
	}
	return ss.str();
}
string AgentModel::executeSkillMove2()
{
	ss.str("");
	cout << "Phase: " << faza << " " << GetJointAngleDeg(JID_LLEG_1) << "  " << GetJointAngleDeg(JID_RLEG_1) << "  " << GetJointAngleDeg(JID_LLEG_2) << "  " << GetJointAngleDeg(JID_RLEG_2) << "  " <<GetJointAngleDeg(JID_LLEG_3) << "  " << GetJointAngleDeg(JID_RLEG_3) << "\n";	
	if(faza == 1)
	{
		if(GetJointAngleDeg(JID_LLEG_2) > -12)
		{
			ss << "(lle2 -1)(rle1 2)(lre1 -2)";
		}
		else
		{
			ss << "(lle2 0)(rle1 0)(lre1 0)";
			faza = 2;
		}
		if(GetJointAngleDeg(JID_RLEG_3) < 15)
		{
			ss << "(rle3 2)";
		}
		else
		{
			ss << "(rle3 0)";
		}
	}
	if(faza == 2)
	{
		if(GetJointAngleDeg(JID_LLEG_2) < 0)
		{
			ss << "(lle2 1)";
		}
		else
		{
			ss << "(lle2 0)";
			faza = 3;
		}
	}
	if(faza == 3)
	{
		if(GetJointAngleDeg(JID_LLEG_3) > 0)
		{
			ss << "(lle3 -1)";
		}
		else
		{
			ss << "(lle3 0)";
		}
		if(GetJointAngleDeg(JID_RLEG_3) > 0)
		{
			ss << "(rle3 -1)";
		}
		else
		{
			ss << "(rle3 0)";
		}
		if(GetJointAngleDeg(JID_LLEG_3) <= 0 && GetJointAngleDeg(JID_RLEG_3) <= 0)
		{
			faza = 1;
		}
	}
	return ss.str();
}
string AgentModel::executeSquat()
{
	ss.str("");
	cout << "Phase: " << faza << " " << GetJointAngleDeg(JID_LLEG_1) << "  " << GetJointAngleDeg(JID_RLEG_1) << "  " << GetJointAngleDeg(JID_LLEG_2) << "  " << GetJointAngleDeg(JID_RLEG_2) << "  " <<GetJointAngleDeg(JID_LLEG_3) << "  " << GetJointAngleDeg(JID_RLEG_3) << "  " <<GetJointAngleDeg(JID_LLEG_4) << "  " << GetJointAngleDeg(JID_RLEG_4) << "\n";
	if(faza == 1)
	{
		if(GetJointAngleDeg(JID_LARM_1) > -90.0)
		{
			ss << "(lae1 -1.0)(rae1 -1.0)";
		}
		else
		{
			ss << "(lae1 0.0)(rae1 0.0)";
			faza=2;
		}
	}
	if(faza == 2)
	{
		if(GetJointAngleDeg(JID_RLEG_3) < 60)
		{
			ss << "(lle1 1)(rle1 1)(lle3 1)(rle3 1)(lle4 -2)(rle4 -2)(lle5 1)(rle5 1)";
		}
		else
		{
			ss << "(lle1 0)(rle1 0)(lle3 0)(rle3 0)(lle4 0)(rle4 0)(lle5 0)(rle5 0)";
			faza=3;
		}
	}
	if(faza == 3)
	{
		if(GetJointAngleDeg(JID_RLEG_3) > 0)
		{
			ss << "(lle3 -1)(rle3 -1)(lle4 2)(rle4 2)(lle5 -1)(rle5 -1)";
		}
		else
		{
			ss << "(lle3 0)(rle3 0)(lle4 0)(rle4 0)(lle5 0)(rle5 0)";
			faza=2;
		}
	}
	return ss.str();
}
string AgentModel::executeMoveLeft()
{
	ss.str("");
	cout << "Phase: " << faza << "\n";
	cout << GetJointAngleDeg(JID_HEAD_1) << "  " << GetJointAngleDeg(JID_HEAD_2) << " ";
	cout << GetJointAngleDeg(JID_LARM_1) << "  " << GetJointAngleDeg(JID_RARM_1) << " ";
	cout << GetJointAngleDeg(JID_LARM_2) << "  " << GetJointAngleDeg(JID_RARM_2) << " ";
	cout << GetJointAngleDeg(JID_LARM_3) << "  " << GetJointAngleDeg(JID_RARM_3) << " ";
	cout << GetJointAngleDeg(JID_LARM_4) << "  " << GetJointAngleDeg(JID_RARM_4) << "\n";
	/*
	//cout << GetJointAngleDeg(JID_LLEG_1) << "  " << GetJointAngleDeg(JID_RLEG_1) << "\n";
	cout << GetJointAngleDeg(JID_LLEG_2) << "  " << GetJointAngleDeg(JID_RLEG_2) << " ";
	//cout << GetJointAngleDeg(JID_LLEG_3) << "  " << GetJointAngleDeg(JID_RLEG_3) << "\n";
	//cout << GetJointAngleDeg(JID_LLEG_4) << "  " << GetJointAngleDeg(JID_RLEG_4) << "\n";
	cout << GetJointAngleDeg(JID_LLEG_5) << "  " << GetJointAngleDeg(JID_RLEG_5) << " ";
	cout << GetJointAngleDeg(JID_LLEG_6) << "  " << GetJointAngleDeg(JID_RLEG_6) << "\n";
	*/
	if(faza == 1)
	{
		if(GetJointAngleDeg(JID_LARM_1) > -90.0)
		{
			ss << "(lae1 -1.0)(rae1 -1.0)";
		}
		else
		{
			ss << "(lae1 0.0)(rae1 0.0)";
			faza=2;
		}
	}
	else if(faza == 2)
	{
		if(GetJointAngleDeg(JID_RLEG_2) < 15.0)
		{
			ss << "(lle2 1)(lle6 -1)(rle2 1)(rle6 -1)";
		}
		else
		{
			ss << "(lle2 0)(lle6 0)(rle2 0)(rle6 0)";
			faza=3;
		}
	}
	else if(faza == 3)
	{
		if(GetJointAngleDeg(JID_LLEG_2) < 30.0)
		{
			ss << "(lle2 1)(lle6 -1)";
		}
		else
		{
			ss << "(lle2 0)(lle6 0)";
			faza=4;
		}
	}
	else if(faza == 4)
	{
		if(GetJointAngleDeg(JID_RLEG_2) > -15)
		{
			ss << "(rle2 -1)(rle6 1)";
		}
		else
		{
			ss << "(rle2 0)(rle6 0)";
			faza=5;
		}
	}
	else if(faza == 5)
	{
		if(GetJointAngleDeg(JID_LLEG_2) > 0)
		{
			ss << "(lle2 -1)(lle6 1)";
		}
		else
		{
			ss << "(lle2 0)(lle6 0)";
			faza=6;
		}
	}
	else if(faza == 6)
	{
		if(GetJointAngleDeg(JID_RLEG_2) < 0)
		{
			ss << "(rle2 1)(rle6 -1)";
		}
		else
		{
			ss << "(rle2 0)(rle6 0)";
			faza=7;
		}
	}
	else if(faza == 7)
	{
		executeStabilisation(JID_LLEG_1, "lle1", 0);
		executeStabilisation(JID_RLEG_1, "rle1", 0);
		executeStabilisation(JID_LLEG_2, "lle2", 0);
		executeStabilisation(JID_RLEG_2, "rle2", 0);
		executeStabilisation(JID_LLEG_3, "lle3", 0);
		executeStabilisation(JID_RLEG_3, "rle3", 0);
		executeStabilisation(JID_LLEG_4, "lle4", 0);
		executeStabilisation(JID_RLEG_4, "rle4", 0);
		executeStabilisation(JID_LLEG_5, "lle5", 0);
		executeStabilisation(JID_RLEG_5, "rle5", 0);
		executeStabilisation(JID_LLEG_6, "lle6", 0);
		executeStabilisation(JID_RLEG_6, "rle6", 0);
		if(isStable(JID_LLEG_1, 0) && isStable(JID_RLEG_1, 0) &&
			isStable(JID_LLEG_2, 0) && isStable(JID_RLEG_2, 0) &&
			isStable(JID_LLEG_3, 0) && isStable(JID_RLEG_3, 0) &&
			isStable(JID_LLEG_4, 0) && isStable(JID_RLEG_4, 0) &&
			isStable(JID_LLEG_5, 0) && isStable(JID_RLEG_5, 0) &&
			isStable(JID_LLEG_6, 0) && isStable(JID_RLEG_6, 0))
		{
			faza=2;
		}
	}
	/*
	else if(faza == 2)
	{
		if(GetJointAngleDeg(JID_RLEG_2) < 15.0)
		{
			executeLegJointMove(JID_LLEG_2, JID_LLEG_6, "lle2", "lle6", 1);
			executeLegJointMove(JID_RLEG_2, JID_RLEG_6, "rle2", "rle6", 1);
			//ss << "(rle2 1)(rle6 -1)(lle2 1)(lle6 -1)";
		}
		else
		{
			ss << "(rle2 0)(rle6 0)(lle2 0)(lle6 0)";
			faza=3;
		}
	}
	else if(faza == 3)
	{
		executeStabilisation(JID_RLEG_2, JID_RLEG_6, "rle2", "rle6", 15);
		if(GetJointAngleDeg(JID_LLEG_2) < 30.0)
		{
			executeLegJointMove(JID_LLEG_2, JID_LLEG_6, "lle2", "lle6", 1);
			//ss << "(lle2 1)(lle6 -1)";
		}
		else
		{
			ss << "(lle2 0)(lle6 0)";
			faza=4;
		}
	}
	else if(faza == 4)
	{
		executeStabilisation(JID_LLEG_2, JID_LLEG_6, "lle2", "lle6", 30);
		if(GetJointAngleDeg(JID_RLEG_2) > -15)
		{
			executeLegJointMove(JID_RLEG_2, JID_RLEG_6, "rle2", "rle6", -1);
			//ss << "(rle2 -1)(rle6 1)";
		}
		else
		{
			ss << "(rle2 0)(rle6 0)";
			faza=401;
		}
	}
	*/
	/*
	if(faza == 5)
	{
		if(GetJointAngleDeg(JID_LLEG_3) < 50)
		{
			ss << "(lle1 1)(lle2 1)(lle3 1)(lle4 -2)(lle5 1)(lle6 0.5)(rle2 1)";
		}
		else
		{
			ss << "(lle1 0)(lle2 0)(lle3 0)(lle4 0)(lle5 0)(lle6 0)(rle2 0)";
			faza=6;
		}
	}
	if(faza == 6)
	{
		if(GetJointAngleDeg(JID_RLEG_3) < 50)
		{
			ss << "(rle1 1)(rle3 1)(rle4 -2)(rle5 1)";
		}
		else
		{
			ss << "(rle1 0)(rle3 0)(rle4 0)(rle5 0)";
			faza=7;
		}
	}*/
	/*
	else if(faza == 401)
	{
		executeStabilisation(JID_LLEG_2, JID_LLEG_6, "lle2", "lle6", 30);
		executeStabilisation(JID_RLEG_2, JID_RLEG_6, "rle2", "rle6", -15);
		if(isStable(JID_LLEG_2, JID_LLEG_6, 30) && isStable(JID_RLEG_2, JID_RLEG_6, -15))
		{
			faza=5;
		}
	}
	else if(faza == 5)
	{
		if(GetJointAngleDeg(JID_LLEG_2) > 20)
		{
			executeLegJointMove(JID_LLEG_2, JID_LLEG_6, "lle2", "lle6", -1);
			executeStabilisation(JID_RLEG_2, JID_RLEG_6, "rle2", "rle6", -15);
			//ss << "(lle2 -1)(lle6 1)";
		}
		else if(GetJointAngleDeg(JID_LLEG_2) > 0)
		{
			executeLegJointMove(JID_LLEG_2, JID_LLEG_6, "lle2", "lle6", -1);
			executeStabilisation(JID_RLEG_2, JID_RLEG_6, "rle2", "rle6", -20);
		}
		else
		{
			ss << "(lle2 0)(lle6 0)";
			faza=501;
		}
	}
	else if(faza == 501)
	{
		executeStabilisation(JID_LLEG_2, JID_LLEG_6, "lle2", "lle6", 0);
		executeStabilisation(JID_RLEG_2, JID_RLEG_6, "rle2", "rle6", -15);
		if(isStable(JID_LLEG_2, JID_LLEG_6, 0) && isStable(JID_RLEG_2, JID_RLEG_6, -15))
		{
			faza=6;
		}
	}
	else if(faza == 6)
	{
		executeStabilisation(JID_LLEG_2, JID_LLEG_6, "lle2", "lle6", 0);
		if(GetJointAngleDeg(JID_RLEG_2) < 0)
		{
			executeLegJointMove(JID_RLEG_2, JID_RLEG_6, "rle2", "rle6", 1);
			//ss << "(rle2 1)(rle6 -1)";
		}
		else
		{
			ss << "(rle2 0)(rle6 0)";
			faza=601;
		}
	}
	else if(faza == 601)
	{
		executeStabilisation(JID_LLEG_2, JID_LLEG_6, "lle2", "lle6", 0);
		executeStabilisation(JID_RLEG_2, JID_RLEG_6, "rle2", "rle6", 0);
		executeStabilisation(JID_LLEG_5, JID_RLEG_5, "lle5", "rle5", 0);
		if(isStable(JID_LLEG_2, JID_LLEG_6, 0) && isStable(JID_RLEG_2, JID_RLEG_6, 0) && isStable(JID_LLEG_5, JID_RLEG_5, 0))
		{
			faza=2;
		}
	}
	*/
	cout << "Order: " << ss.str() << "\n";
	return ss.str();
}
void AgentModel::executeStabilisation(JointID jid, string str, double deg)
{
	if(GetJointAngleDeg(jid) < deg-1.5)
	{
		ss << "(" << str << " 1.2)";
	}
	else if(GetJointAngleDeg(jid) < deg-1)
	{
		ss << "(" << str << " 0.8)";
	}
	else if(GetJointAngleDeg(jid) < deg-0.5)
	{
		ss << "(" << str << " 0.4)";
	}
	else if(GetJointAngleDeg(jid) < deg-0.2)
	{
		ss << "(" << str << " 0.2)";
	}
	/*
	else if(GetJointAngleDeg(jid) < deg-0.1)
	{
		ss << "(" << str << " 0.1)";
	}
	*/
	else if(GetJointAngleDeg(jid) > deg+1.5)
	{
		ss << "(" << str << " -1.2)";
	}
	else if(GetJointAngleDeg(jid) > deg+1)
	{
		ss << "(" << str << " -0.8)";
	}
	else if(GetJointAngleDeg(jid) > deg+0.5)
	{
		ss << "(" << str << " -0.4)";
	}
	else if(GetJointAngleDeg(jid) > deg+0.2)
	{
		ss << "(" << str << " -0.2)";
	}
	/*
	else if(GetJointAngleDeg(jid) > deg+0.1)
	{
		ss << "(" << str << " -0.1)";
	}
	*/
	else
	{
		ss << "(" << str << " 0)";
	}
}
bool AgentModel::isStable(JointID jid, double deg)
{
	return (GetJointAngleDeg(jid)>=deg-0.2 && GetJointAngleDeg(jid)<=deg+0.2);
}
string AgentModel::executeSkillRun()
{
	ss.str("");
	//cout << "Phase: " << faza << " " << GetJointAngleDeg(JID_LLEG_1) << "  " << GetJointAngleDeg(JID_RLEG_1) << "  " << GetJointAngleDeg(JID_LLEG_2) << "  " << GetJointAngleDeg(JID_RLEG_2) << "  " <<GetJointAngleDeg(JID_LLEG_3) << "  " << GetJointAngleDeg(JID_RLEG_3) << "\n";	
	if(faza==1){
		if(GetJointAngleDeg(JID_LLEG_4) > -40){
			ss<<"(rle4 -1)(lle4 -1)(rle3 0.5)(lle3 0.5)(lle5 0.5)(rle5 0.5)";
		}else{
			faza=2;
			cout<<"ENTERING PHASE:"<<faza<<"\n";
			return "(rle4 0)(lle4 0)(rle3 0)(lle3 0)(lle5 0)(rle5 0)";
		}
	}
	if(faza==2){
		if(GetJointAngleDeg(JID_RLEG_4) > -80){
			ss<<"(rle4 -0.5)(rle3 0.25)(rle5 0.25)(rle2 -0.25)(lle2 -0.25)(rle6 -0.075)";
		}else{
			faza=4;
			cout<<"ENTERING PHASE:"<<faza<<"\n";
			return "(rle4 0)(lle4 0)(rle3 0)(lle3 0)(lle5 0)(rle5 0)(rle2 0)(lle2 0)(rle6 0)";
		}
	}
	if(faza==3){
		if(GetJointAngleDeg(JID_RLEG_5) < 40){
			ss<<"(rle5 0.5)(lle3 1)(rle3 -0.75)";
		}else{
			ss<<"(rle5 0)(lle3 0)(rle3 -0)";
			faza=4;
			cout<<"ENTERING PHASE:"<<faza<<"\n";
			return ss.str();
		}
	}
	
	if(faza==4){
		
		if(GetJointAngleDeg(JID_RLEG_4) < -15){
			ss<<"(rle4 0.15)(rle3 -0.15)(lle3 0.01)(lle4 0.05)(rle2 0.02)(lae1 -0.35)(lle6 0.01)(lle5 -0.05)";
		}else{
			faza=5;
			cout<<"ENTERING PHASE:"<<faza<<"\n";
			return "(rle4 00)(rle3 0)(lle3 0)(lle4 0)(rle2 0)(lae1 0)(lle6 0)(lle5 0)";
		}
	}
	if(faza==5){
		//cout<<GetJointAngleDeg(JID_LLEG_4)<<"/n";
		if(GetJointAngleDeg(JID_LLEG_4) > -70){
			ss<<"(lle4 -0.1)(lle3 0.05)(lle5 0.05)(lle2 0.05)(rle2 0.05)";
		}else{
			faza=6;
			cout<<"ENTERING PHASE:"<<faza<<"\n";
			return "(lle4 0)(lle3 0)(lle5 0)(rle3 0)(lle4 0)(lle2 0)(rle2 0)(rae1 0)(rle6 0)(rle5 0)";
		}
	}	

	if(faza==6){
		//cout<<GetJointAngleDeg(JID_LLEG_4)<<"/n";
		if(GetJointAngleDeg(JID_LLEG_4) < -20){
			ss<<"(lle4 0.15)(lle3 -0.15)(rle3 0.05)(rle4 0.1)(lle2 0.02)(rle2 0.01)(rae1 -0.2)(rle6 0.015)(rle5 -0.08)";
		}else{
			faza=7;
			cout<<"ENTERING PHASE:"<<faza<<"\n";
			return "(lle4 00)(lle3 0)(rle3 0)(lle4 0)(lle2 0)(rle2 0)(rae1 0)(rle6 0)(rle5 0)";
		}
	}
	if(faza==7){
		
		if(GetJointAngleDeg(JID_LLEG_4) > -60){
			ss<<"(lle4 -0.15)(lle3 -0.15)(lle5 0.15)(rle3 0.1)(rle4 -0.15)(lle2 0.005)(rle2 0.02)(rae1 -0.2)(rle6 -0.02)(rle5 -0.05)(lae 0.2)";
		}else{
			faza=8;
			cout<<"ENTERING PHASE:"<<faza<<"\n";
			return "(lle4 00)(lle3 0)(lle5 0)(rle3 0)(lle4 0)(rle4 0)(lle2 0)(rle2 0)(rae1 0)(rle6 0)(rle5 0)(lae 0)";
		}
	}

	if(faza==8){
		//cout<<GetJointAngleDeg(JID_LLEG_4)<<"/n";
		if(GetJointAngleDeg(JID_RLEG_4) > -80){
			ss<<"(rle4 -0.1)(rle3 0.1)(lle3 0.1)(rle5 0.05)(rle2 -0.05)(lle2 -0.05)(lle6 -0.01)(rle6 -0.01)(lae1 0.2)";
		}else{
			faza=10;
			cout<<"ENTERING PHASE:"<<faza<<"\n";
			return "(rle4 0)(rle3 0)(rle5 0)(lle3 0)(rle4 0)(rle2 0)(lle2 0)(rae1 0)(lae1 0)(rle6 0)(lle6 0)(rle5 0)(lle5 0)";
		}
	}
	//if(faza==9){
	//	cout<<GetJointAngleDeg(JID_LLEG_3)<<"\n";
	//	if(GetJointAngleDeg(JID_RLEG_3) < 20){
	//		ss<<"(rle3 0.2)(lle3 0.2)";
	//	}else{
	//		faza=10;
	//		cout<<"ENTERING PHASE:"<<faza<<"\n";
	//		return "(rle4 0)(rle3 0)(rle5 0)(lle3 0)(rle4 0)(rle2 0)(lle2 0)(lae1 0)(lle6 0)(lle5 0)";
	//	}
	//}	
	if(faza==10){
		//cout<<GetJointAngleDeg(JID_LLEG_4)<<"/n";
		if(GetJointAngleDeg(JID_RLEG_4) < -30){
			ss<<"(rle4 0.2)(rle3 -0.15)(lle3 0.05)(lle4 0.1)(rle2 0.02)(lle2 0.01)(lae1 -0.2)(lle6 0.015)(lle5 -0.2)";
		}else{
			faza=5;
			cout<<"ENTERING PHASE:"<<faza<<"\n";
			return "(rle4 00)(lle4 0)(rle3 0)(lle3 0)(rle4 0)(rle2 0)(lle2 0)(lae1 0)(lle6 0)(lle5 0)";
		}
	
	}
	/*if(faza==8){
		cout<<GetJointAngleDeg(JID_RLEG_5)<<"FAZA 7\n";
		if(GetJointAngleDeg(JID_RLEG_5) > -50){
			ss<<"(lle4 -0.15)(lle3 -0.15)(lle5 0.15)(rle3 0.1)(rle4 -0.15)(lle2 0.005)(rle2 0.02)(rae1 -0.2)(rle6 0.015)(rle5 -0.02)";
		}else{
			faza=10;
			return "(lle4 00)(lle3 0)(lle5 0)(rle3 0)(lle4 0)(rle4 0)(lle2 0)(rle2 0)(rae1 0)(rle6 0)(rle5 0)";
		}
	}*/
	//if(faza == 2)
	//{
	//	if(GetJointAngleDeg(JID_RLEG_3) > -1)
	//	{
	//		cout << GetJointAngleDeg(JID_RLEG_3)<< "\n";
	//		//
	//		ss << "(rle3 -0.1)(lle3 0.1)";
	//	}else {
	//		faza=3;
	//	}
	//}
	//if(faza == 3)
	//{
	//	if(GetJointAngleDeg(JID_RLEG_6) < 10)
	//	{
	//		//(lle2 0.5)(rle2 -0.5)(lle3 -2)
	//		ss << "(rle6 0.1)";
	//	}else {
	//		faza=1;
	//	}
	//}
	//
	/*if(faza==7){
			ss<<"(rle4 0)(lle4 0)(rle3 0)(lle3 0)(lle5 0)(rle5 0)";
	}
	if(faza==8){
		ss<<"(rle4 0)(lle4 0)(rle3 0)(lle3 0)(lle5 0)(rle5 0)";
	}*/
	return ss.str();
}

string AgentModel::executeSkillDrepuj()
{
	cout << GetJointAngleDeg(JID_RLEG_4)<< "  FAZA:"<<faza<<"\n";
	if(faza==1){
		if(GetJointAngleDeg(JID_RLEG_4) > -110){
			ss<<"(rle4 -1)(lle4 -1)(rle3 0.5)(lle3 0.5)(lle5 0.5)(rle5 0.5)";
		}else{
			faza=2;
			//return "(rle4 0)(lle4 0)(rle3 0)(lle3 0)(lle5 0)(rle5 0)";
		}
	}
	if(faza==2){
		if(GetJointAngleDeg(JID_RLEG_4) < 0){
			ss<<"(rle4 1)(lle4 1)(rle3 -0.5)(lle3 -0.5)(lle5 -0.5)(rle5 -0.5)";
		}else{
			faza=3;
			//return "(rle4 0)(lle4 0)(rle3 0)(lle3 0)(lle5 0)(rle5 0)";
		}
	}
	if(faza==3){
		ss<<"(rle3 0.5)(lle3 0.5)(lle5 0.6)(rle5 0.6)";
		faza=1;
	}
	return ss.str();
}
// TODO: Lukas Move to alone class
string AgentModel::executeFallDown()
{
	if (step_1 == 0) {
		ss.str("");
		if (type_1 == 0) {
			cout << "executeFallDown: type="<<type_1 << " : "<<GetJointAngleDeg(JID_LARM_1)<<"  "<<GetJointAngleDeg(JID_RARM_1) <<"\n" ;
			if (GetJointAngleDeg(JID_LARM_1) > -90.0) {
				ss << "(lae1 -1.0)";
			}
			
			if (GetJointAngleDeg(JID_RARM_1) > -90.0) {
				ss << "(rae1 -1.0)";
			}
			

			if (GetJointAngleDeg(JID_LARM_1) <= -90.0 && GetJointAngleDeg(JID_RARM_1) <= -90.0) {
				ss << "(lae1 0.0)";
				ss << "(rae1 0.0)";
				type_1 =1;
			}
		} 
		if (type_1 == 1) {
			cout << "executeFallDown: type="<<type_1 << " : "<<GetJointAngleDeg(JID_LLEG_5)<<"  "<<GetJointAngleDeg(JID_RLEG_5) <<"\n" ;
			if (GetJointAngleDeg(JID_LLEG_5) > -30.0) {
				ss << "(lle5 -1.0)";
			}
			
			if (GetJointAngleDeg(JID_RLEG_5) > -30.0) {
				ss << "(rle5 -1.0)";
			}
			

			if (GetJointAngleDeg(JID_LLEG_5) <= -30.0 && GetJointAngleDeg(JID_RLEG_5) <= -30.0) {
				ss << "(lae1 0.0)";
				ss << "(rae1 0.0)";
				type_1 = 0;
				step_1 = 1;
			}
		}
		return ss.str();
	} else {
		return moveRoundOnEarth();
	}
	
	
}

string AgentModel::moveRoundOnEarth()
{
	ss.str("");
	if (type_1 == 0) {
		cout << "moveRoundOnEarth: type="<<type_1 << " : "<<GetJointAngleDeg(JID_LLEG_5)<<"  "<<GetJointAngleDeg(JID_RLEG_5) <<"\n" ;
		if (GetJointAngleDeg(JID_LARM_1) > -90.0) {
			ss << "(lae1 -1.0)";
		}
		
		if (GetJointAngleDeg(JID_RARM_1) > -90.0) {
			ss << "(rae1 -1.0)";
		}
		

		if (GetJointAngleDeg(JID_LARM_1) <= -90.0 && GetJointAngleDeg(JID_RARM_1) <= -90.0) {
			ss << "(lae1 0.0)";
			ss << "(rae1 0.0)";
			type_1 =1;
		}
	} 
	if (type_1 == 1) {
		
		if (GetJointAngleDeg(JID_LLEG_5) > -90.0) {
			ss << "(lle5 -1.0)";
		}
		
		if (GetJointAngleDeg(JID_RLEG_5) > -90.0) {
			ss << "(rle5 -1.0)";
		}
		

		if (GetJointAngleDeg(JID_LLEG_5) <= -90.0 && GetJointAngleDeg(JID_RLEG_5) <= -90.0) {
			ss << "(lae1 0.0)";
			ss << "(rae1 0.0)";
			type_1 =1;
		}
	} 
	
	return ss.str();
}


