#include "WorldModel.h"
#include "../../../config/Config.h"
#include <queue>

extern Config *config;
#define DELETE_HEAR_AFTER_SECONDS 10
#define MAX_SEE_PLAYERS_ON_ONE_SIDE 10

WorldModel::WorldModel()
{
	ball = new Ball();

	ourNet = new Net();
	opponentNet = new Net();

	ourLeftCorner = new Object();
	ourRightCorner = new Object();
	opponentLeftCorner = new Object();
	opponentRightCorner = new Object();
	hearCount = 0;
	opponentsCount = 0;
	teammatesCount = 0;
}

void WorldModel::update(ServerMsgParserData *parsedData)
{
	simTime = parsedData->GetSimTime();
	gameTime = parsedData->GetGameTime();
	playMode = parsedData->GetPlayMode();

	ball->position = parsedData->GetBallPosition();
	ball->timestamp = simTime;

	ourNet->left.position = parsedData->GetFlagPosition(OBJ_FLAG_G1L);
	ourNet->right.position = parsedData->GetFlagPosition(OBJ_FLAG_G2L);
	opponentNet->left.position = parsedData->GetFlagPosition(OBJ_FLAG_G1R);
	opponentNet->right.position = parsedData->GetFlagPosition(OBJ_FLAG_G2R);

	ourLeftCorner->position = parsedData->GetFlagPosition(OBJ_FLAG_F1L);
	ourRightCorner->position = parsedData->GetFlagPosition(OBJ_FLAG_F2L);
	opponentLeftCorner->position = parsedData->GetFlagPosition(OBJ_FLAG_F1R);
	opponentRightCorner->position = parsedData->GetFlagPosition(OBJ_FLAG_F2R);

	HearPerceptor *tempHear = parsedData->GetHear();
	hearCount = parsedData->GetHearCount();

	if(hearCount != 0)
	{
		for(int i=0;i<hearCount;i++)
		{
			hear[i].direction = tempHear[i].direction;
			hear[i].hearTime = tempHear[i].hearTime;
			hear[i].myself = tempHear[i].myself;
	#ifdef WIN32
			strcpy_s(hear[i].message,MAX_MESSAGE_SIZE,tempHear[i].message);
	#else
			strcpy(hear[i].message,MAX_MESSAGE_SIZE,tempHear[i].message,strlen(tempHear[i].message));
	#endif
		}
	}

	//nulovanie hear perceptora, nepotrebujeme mazat vsetko, pretoze sa to aj tak prepisuje
	//if((int)simTime % DELETE_HEAR_AFTER_SECONDS == 0)
	//{
	//	hearCount = 0;
	//}

	Player *tempPlayers = parsedData->GetPlayers();
	int tempPlayersCount = parsedData->GetPlayersCount();

	if(tempPlayersCount != 0)
	{
		for(int i=0;i<tempPlayersCount;i++)
		if(strcmp(tempPlayers[i].playerTeam,config->player->team))
		{
			//ak je to protihrac
			UpdatePlayer(tempPlayers[i],opponents,&opponentsCount);
			//opponentsCount++;
		}
		else
		{
			//ak je to spoluhrac
			UpdatePlayer(tempPlayers[i],teammates,&teammatesCount);
			//teammatesCount++;
		}
	}

	//cout << "Vidim nasich: " << teammatesCount << endl;
	//cout << "Vidim superovych: " << opponentsCount << endl << endl;
}

void WorldModel::UpdatePlayer(Player seePlayer,Player teamPlayers[],int *playerCount)
{
	bool found = false;
	int iter = 0;
	do
	{
		//ak sa v poli nenachadza ziaden hrac, pridaj ho
		if(*playerCount == 0)
		{
			seePlayer.timestamp = simTime;
			teamPlayers[*playerCount] = seePlayer;
			*playerCount += 1;
			found = true;
			break;
		}
		else
			//ak sa v zozname nachadza hrac s danym ID, aktualizuj mu parametre
			if(teamPlayers[iter].id == seePlayer.id)
			{
				teamPlayers[iter].position = seePlayer.position;
				teamPlayers[iter].timestamp = simTime;
				found = true;
			}
		iter++;
	} 
	while (iter < *playerCount && found != true);

	//ak sa v zozname nenasiel hrac, pridaj ho
	if(!found)
	{
		seePlayer.timestamp = simTime;
		teamPlayers[iter] = seePlayer;
		*playerCount += 1;
	}
}

WorldModel::~WorldModel()
{
	delete ball;
	delete ourNet;
	delete opponentNet;
	delete ourLeftCorner;
	delete ourRightCorner;
	delete opponentLeftCorner;
	delete opponentRightCorner;
}