#ifndef JOINTS_H
#define JOINTS_H

#include <map>

struct HingeJointSense
{
	/** joint angle */
	double angle;

	/** joint angle rate */
	double rate;

	HingeJointSense() : angle(0), rate(0) {};
};

enum JointID
{
	JID_HEAD_1, 
	JID_HEAD_2,

	JID_LLEG_1,
	JID_RLEG_1,
	JID_LLEG_2,
	JID_RLEG_2,
	JID_LLEG_3,
	JID_RLEG_3,
	JID_LLEG_4,
	JID_RLEG_4,
	JID_LLEG_5,
	JID_RLEG_5,
	JID_LLEG_6,
	JID_RLEG_6,

	JID_LARM_1,
	JID_RARM_1,
	JID_LARM_2,
	JID_RARM_2,
	JID_LARM_3,
	JID_RARM_3,
	JID_LARM_4,
	JID_RARM_4,
};

// mapping from joint id to joint hinge sense object
typedef std::map<JointID, HingeJointSense> THingeJointSenseMap;

// mapping from object name to joint id
typedef std::map<std::string, JointID> TJointIDMap;

template <class T>
T SetupJointIDMap(T tJointIDMap)
{
	tJointIDMap.clear();
	tJointIDMap["hj1"]      = JID_HEAD_1;
	tJointIDMap["hj2"]      = JID_HEAD_2;

	tJointIDMap["llj1"]     = JID_LLEG_1;
	tJointIDMap["rlj1"]     = JID_RLEG_1;
	tJointIDMap["llj2"]     = JID_LLEG_2;
	tJointIDMap["rlj2"]     = JID_RLEG_2;
	tJointIDMap["llj3"]     = JID_LLEG_3;
	tJointIDMap["rlj3"]     = JID_RLEG_3;
	tJointIDMap["llj4"]     = JID_LLEG_4;
	tJointIDMap["rlj4"]     = JID_RLEG_4;
	tJointIDMap["llj5"]     = JID_LLEG_5;
	tJointIDMap["rlj5"]     = JID_RLEG_5;
	tJointIDMap["llj6"]     = JID_LLEG_6;
	tJointIDMap["rlj6"]     = JID_RLEG_6;

	tJointIDMap["laj1"]     = JID_LARM_1;
	tJointIDMap["raj1"]     = JID_RARM_1;
	tJointIDMap["laj2"]     = JID_LARM_2;
	tJointIDMap["raj2"]     = JID_RARM_2;
	tJointIDMap["laj3"]     = JID_LARM_3;
	tJointIDMap["raj3"]     = JID_RARM_3;
	tJointIDMap["laj4"]     = JID_LARM_4;
	tJointIDMap["raj4"]     = JID_RARM_4;

	return tJointIDMap;
}
template <class X>
X SetupJointIDMapWithEfectors(X tJointIDMap)
{
	tJointIDMap.clear();
	tJointIDMap["he1"]      = JID_HEAD_1;
	tJointIDMap["he2"]      = JID_HEAD_2;

	tJointIDMap["lle1"]     = JID_LLEG_1;
	tJointIDMap["rle1"]     = JID_RLEG_1;
	tJointIDMap["lle2"]     = JID_LLEG_2;
	tJointIDMap["rle2"]     = JID_RLEG_2;
	tJointIDMap["lle3"]     = JID_LLEG_3;
	tJointIDMap["rle3"]     = JID_RLEG_3;
	tJointIDMap["lle4"]     = JID_LLEG_4;
	tJointIDMap["rle4"]     = JID_RLEG_4;
	tJointIDMap["lle5"]     = JID_LLEG_5;
	tJointIDMap["rle5"]     = JID_RLEG_5;
	tJointIDMap["lle6"]     = JID_LLEG_6;
	tJointIDMap["rle6"]     = JID_RLEG_6;

	tJointIDMap["lae1"]     = JID_LARM_1;
	tJointIDMap["rae1"]     = JID_RARM_1;
	tJointIDMap["lae2"]     = JID_LARM_2;
	tJointIDMap["rae2"]     = JID_RARM_2;
	tJointIDMap["lae3"]     = JID_LARM_3;
	tJointIDMap["rae3"]     = JID_RARM_3;
	tJointIDMap["lae4"]     = JID_LARM_4;
	tJointIDMap["rae4"]     = JID_RARM_4;

	return tJointIDMap;
}

#endif //JOINTS_H