#include "Dispatcher.h"
#include <cstdlib>
#include <vector>

using namespace std;

Dispatcher::Dispatcher()
{
	XMLSkillsFactory factory;
	forceReplan = false;
	skills = *factory.getAllHighSkills();
	activeSkill = NULL;
	//activeSkill = new Skill_8();
}

Dispatcher::~Dispatcher(void)
{
	delete &skills;
	delete &forceReplan;
	delete activeSkill;
}

HighSkill *Dispatcher::replanAction(States *states)
{
	return replanAction(states, false);
}

HighSkill *Dispatcher::replanAction(States *states, bool excludeActiveSkill)
{
	HighSkill *nextSkill = NULL;
	HighSkill *curSkill = NULL;
	HighSkill *preCurSkill = NULL;
	
	for(unsigned i=0;i<skills.size();i++)
	{
		nextSkill = skills[i];

		//don't let to choose activeSkill again ...
		if(activeSkill == nextSkill)
		{
			continue;
		}

		preCurSkill = nextSkill->compareUsability(curSkill, states);

		if(preCurSkill == NULL)
		{
			preCurSkill = decideBetweenEqualSkills(nextSkill, curSkill);
		}

		curSkill = preCurSkill;
	}

	this->activeSkill = curSkill;

	return curSkill;
}

HighSkill *Dispatcher::decideBetweenEqualSkills(HighSkill *skillA, HighSkill *skillB)
{
	//barti: here can be added some tactics - f. ex.: defensive, or offensive behaviour
	return chooseRandomSkill(skillA, skillB);
}

HighSkill *Dispatcher::chooseRandomSkill(HighSkill *skillA, HighSkill *skillB)
{
	if((rand()%2)==0)
	{
		return skillA;
	}
	else
	{
		return skillB;
	}
}

void Dispatcher::addSkill(HighSkill *skill)
{
	skills.push_back(skill);
}


void Dispatcher::overrideCurrentAction()
{
}

void Dispatcher::performCurrentAction(States *states)
{
	this->activeSkill->execute(states);
}

void Dispatcher::forceReplanAction()
{
	forceReplan = true;
}

void Dispatcher::proceed(States *states)
{
	if(activeSkill == NULL || activeSkill->hasFinished() || (forceReplan && activeSkill->isInReplanableState()))
	{
		forceReplan = false;
		replanAction(states);
	}
	
	try
	{
		performCurrentAction(states);
	}
	catch(IllegalStateException)
	{
		replanAction(states, true);
	}
}