#pragma once

#include "../WorldModel.h"
#include "../../../../config/soccertypes.h"
#include <iostream>
#include <cmath>
#include <vector>

using namespace std;

enum BallHolders {OUR_TEAM, OPPONENT_TEAM};
enum TeamPosition {GOALIE, DEFENDER, ATTACKER, HALF_BACK};
enum KickPossibility {NOT_POSSIBLE, RIGHT_FRONT, RIGHT_SIDE, RIGHT_BACK, LEFT_FRONT, LEFT_SIDE, LEFT_BACK};

const double ballTouchDistance = 1.0;
const double maxDistance = 100;
const double freeDistanceFromPath = 2;
const double angle = 5;
const double frontKickAngle = 10;
const double kickAngleTolerance = 5;

class States
{
private:

	WorldModel* worldModel;

	TPlayMode playMode;

	bool onGround;

	TeamPosition teamPosition;

	KickPossibility kickPossibility;

	vector<Player> freeTeamMates;

	BallHolders ballHolder;
	bool ballNearMe;
	bool goalFree;

	void updateBallHolder();
	void updateBallNearMe();
	void updateGoalFree();
	void updateOnGround();
	void updateKickPossibility();
	void updateFreeTeamMates();

	bool isInHorizontalAngle(const CoordSpherical & coords, double angle, double tolerance);
	bool isInVerticalAngle(const CoordSpherical & coords, double angle, double tolerance);
	bool isInAngle(double testAngle, double expectedAngle, double tolerance);

	vector<CoordSpherical> getBlockingObjectsCoords();
	bool isBetweenMeAndObject(const CoordSpherical & blockingObjectCoords, const CoordSpherical & destObjectCoords);
	bool isPathFreeToObject(const Object * destObject);
	bool isPathFreeToPosition(const CoordSpherical coords);

	double getAngleBetweenObjects(Object * object1, Object * object2);
	CoordSpherical getNearestCorner();
public:
	States(WorldModel* worldModel);

	void update();

	void setTeamPosition(TeamPosition _teamPosition);
	TeamPosition getTeamPosition();

	bool isOnGround();

	bool isGoalFree();

	KickPossibility getKickPossibility();

	vector<Player> getFreeTeamMates();

	double getDistance(const CoordSpherical & from, const CoordSpherical & to);
	double getDistance(const CoordCartesian & from, const CoordCartesian & to);
	double getDistance(const Object & from, const Object & to);
	double getDistanceFromMe(const CoordSpherical & target);
	double getDistanceFromMe(const Object & target);

	~States();
};
