#include "LowSkill.h"

string LowSkill::getName() {
	return name;
}


LowSkill::LowSkill(MovementPhase * initialPhase) {
	performInit = true;
	this->name = "no name specified";
}

LowSkill::LowSkill(string & name, MovementPhase * initialPhase) {
	performInit = true;
	this->name = string(name);
	this->activePhase=initialPhase;
	this->finalizeFlag=false;
}

LowSkill::LowSkill(string & name, string & initialPhase) {
	performInit = true;
	this->name = name;
	this->finalizeFlag=false;
	PhaseCache * cache=PhaseCache::getInstance();
	this->activePhase=new MovementPhase(AgentModel::getInstance(), &cache->get(initialPhase));
}

/**
*	performs execution of low skill. This method should be called in every phase.
*
*/
void LowSkill::execute()
{
	if(performInit){
		//Start the execution-send data to server
		start();
		performInit = false;
	}

	// Perfom checking - idea is that no data sending is required if everything goes well.
	//Data should be sent directly from MovementPhase to Agent wihtout backFlow. Only control flow shall flow back.
	check();
}

/**
	This method should be called in every cycle to track execution
	Returns true if execution continues. If finished returns false;
*/
bool LowSkill::check(){
	try{
		if(!activePhase->check()){//Check if the phase has ended
			MovementPhase * old=activePhase;
			//If yes we have to decide how to continue or whether to finish.
			if(finalizeFlag){
				if(activePhase->isFinal()){
					activePhase->interrupt();
					return false;//Execution complete
				}
				//Move to next finalization phase
				activePhase=activePhase->getFinalizationPhase();
			}else{
				activePhase=activePhase->getNextPhase();
			}
			delete old;
			activePhase->initialize();
			activePhase->proceed();
		}
	}catch(exception* e){
		//Something goes wrong, let's save it:)
		MovementPhase * old=activePhase;
		activePhase=activePhase->calculateRescuePhase();
		if(old!=activePhase){
			activePhase->initialize();
			delete old;
		}
		activePhase->proceed();
		delete e;
	}
	return true;
}
/**
	This method starts execution of Phase.
*/
void LowSkill::start(){
	activePhase->initialize();
	activePhase->proceed();//Force sending data to server.
}
/**
	End the LowSkill
*/
void LowSkill::finalize(){
	finalizeFlag=true;
}
/**
	Pause the LowSkill
*/
void LowSkill::interrupt(){
	activePhase->interrupt();
}
/**
	Continue execution
*/
void LowSkill::proceed(){
	activePhase->proceed();
}
LowSkill::~LowSkill() {

}
