#include "MovementPhase.h"



MovementPhase::MovementPhase(AgentModel * model, PhaseData * data)
{
	this->data=data;
	this->anglesPrevExecution.reserve(data->efectorsData.size());
	this->speedConstant=(((double)data->getSpeedConstant())/1000);
	this->model=model;
}

MovementPhase::~MovementPhase(void)
{
}

/**
	This method prepares Phase for execution: sets movement speeds and updates previous
	angles to by able to perform Increment Check.
*/
void MovementPhase::initialize(){
	//Count angle movement speeds for each joint effector used during this phase
	cout <<"initializing" << data->getName()<<"\n";
	unsigned int size = data->efectorsData.size();

	for(unsigned int i=0;i<size;i++){
		EfectorData * ed= &data->efectorsData.at(i);
		ed->movementSpeed= (ed->getEnd()-model->GetJointAngleDeg(ed->getJointID()))*speedConstant;	
	}
	updatePreviousAngles();
}

/**
	Checks if this phase is still active. 
	returns true if this phase continues.
	returns false if it has ended.
	throws exception if an undesirable state has ben reached.
*/
bool MovementPhase::check(){
	if(checkFinished()){
		cout << "finished "<<data->getName()<< "\n";
		interrupt();
		return false;
	}
	
	//if(!checkIncrement()){
	//	//cout<< "incremet too small";
	//	throw new exception("Joint angle increment too small!");
	//}
	updatePreviousAngles();
	return true;
}
/**
	Check for the end state of the phase
	returns true if finished.
	returns false if stil executing.
*/
bool MovementPhase::checkFinished(){
	for(unsigned int i=0;i<data->efectorsData.size();i++){
		EfectorData * ed= &data->efectorsData.at(i);
		//cout<<model->GetJointAngleDeg(ed->getJointID());
		
		// prev: abs(ed->getEnd())<=model->GetJointAngleDeg(ed->getJointID())*sign(ed->movementSpeed)
		if((ed->movementSpeed>=0 && ed->getEnd()<=model->GetJointAngleDeg(ed->getJointID()))
			|| (ed->movementSpeed<0 && ed->getEnd()>=model->GetJointAngleDeg(ed->getJointID()))) {
			//Desired angle reached
			return true;//Naive implementation. Probbably needs better implementation.
		}
	}
	//cout<< "\n";
	return false;
}

/**
	Checks if all joint angles are incremented.
	Note comprarison with movementSpeed resolved by experiments.
	returns false if the increment was too small.
*/
bool MovementPhase::checkIncrement(){
	for(unsigned int i=0;i<data->efectorsData.size();i++){
		EfectorData * ed= &data->efectorsData.at(i);
		//cout<<abs(anglesPrevExecution.at(i)-model->GetJointAngleDeg(ed->getJointID()))<<"\n";
		if(abs(anglesPrevExecution.at(i)-model->GetJointAngleDeg(ed->getJointID()))<ed->movementSpeed/1.2){
			return false;
		}
	}
	return true;
}
/**
	Stores currently delivered agentModel for incremetChecking in next check.
*/
void MovementPhase::updatePreviousAngles(){
	anglesPrevExecution.clear();
	for(unsigned int i=0;i<data->efectorsData.size();i++){
		anglesPrevExecution.push_back(model->GetJointAngleDeg(data->efectorsData.at(i).getJointID()));
	}
}

/**
	Method to stop all joint movement, but leaving previous movement speed and configuration untouched
*/
void MovementPhase::interrupt(){
	stringstream ss;
	for(unsigned int i=0;i<data->efectorsData.size();i++){
		EfectorData * ed= &data->efectorsData.at(i);
		ss << "(" << ed->getName()<<" 0)";
	}
	Agent::getInstance()->addToMessage(ss.str());
}
/**
	Called to resume movement after being stopped by interrupt method.
*/
void MovementPhase::proceed(){
	sendMessage();//Probbably not the best solution but could be enough.
}
/**
	This method builds communication message and sends it to server
*/
void MovementPhase::sendMessage(){
	stringstream ss;
	for(unsigned int i=0;i<data->efectorsData.size();i++){
		EfectorData * ed= &data->efectorsData.at(i);
		ss << "(" << ed->getName() << " "<< ed->movementSpeed <<")";
	}
	Agent::getInstance()->addToMessage(ss.str());
	cout << ss.str();
}

MovementPhase * MovementPhase::calculateRescuePhase(){
	string s=data->getRescueMovement();
	if(s=="ROLLBACK"){
		PhaseData * d= new PhaseData(data->getName()+"-rollback",data->getSpeedConstant());
		for(unsigned int i=0;i<data->efectorsData.size();i++){
			EfectorData * ed= &data->efectorsData.at(i);
			d->addEfectorData(*new EfectorData(ed->getName(),ed->getEnd(),ed->getStart()));
		}
		d->setFinalizationPhase(data->getName());
		d->setNext(data->getName());
		d->setRescueMovement(string("PROCEED"));
		return new MovementPhase(model,d);
	}
	if(s=="PROCEED"){
		return this;
	}
	return new MovementPhase(model, &PhaseCache::getInstance()->get(s));
}

MovementPhase * MovementPhase::getFinalizationPhase(){
	return new MovementPhase(model, &PhaseCache::getInstance()->get(data->getFinalizationPhase()));
}

MovementPhase * MovementPhase::getNextPhase(){
	return new MovementPhase(model, &PhaseCache::getInstance()->get(data->getNext()));
}

bool MovementPhase::isFinal(){
	return data->isFinal();
}

int MovementPhase::sign(double &d){
	if(d>=0){
		return 1;
	}
	return -1;
}

