#include "XMLSkillsFactory.h"

HighSkill *XMLSkillsFactory::getHighSkillInstance(string name)
{
	if(name == string("walk_to_ball")){
		return new GoToPointHighSkill(name);
	}
	else if(name == string("kick_ball"))
	{
		return new KickHighSkill(name);
	}
	else if(name == string("stand_up"))
	{
		return new StandUpHighSkill(name);
	}
	else if(name == string("turn_to_ball"))
	{
		return new TurnToBallHighSkill(name);
	}
	else if(name == string("walking_back"))
	{
		return new WalkBackHighSkill(name);
	}

	return NULL;
}

XMLSkillsFactory::XMLSkillsFactory() {
	xmlSkillsFilePath = "xml\\efectors.xml";
	time(&lastTimeModified);
	phaseCache = PhaseCache::getInstance();

}

vector<HighSkill*> * XMLSkillsFactory::getAllHighSkills() {

	if (parsingNeeded()) {
		prepare();
		parse();
	}

	return &highSkills;
}

bool XMLSkillsFactory::parsingNeeded() {
	struct stat xmlFileStat;
	stat(xmlSkillsFilePath, &xmlFileStat);
	if (difftime(lastTimeModified, xmlFileStat.st_mtime) != 0) {
		lastTimeModified = xmlFileStat.st_mtime;
		return true;
	} else {
		return false;
	}
}

void XMLSkillsFactory::prepare() {
	phaseCache->clear();
	lowSkills.clear();
	highSkills.clear();
}

int XMLSkillsFactory::parse() {
	XMLPlatformUtils::Initialize();

	XercesDOMParser * parser = new XercesDOMParser();
	parser->setValidationScheme(XercesDOMParser::Val_Always);
	ErrorHandler * errorHandler = (ErrorHandler *) new HandlerBase();
	parser->setErrorHandler(errorHandler);
	parser->setIncludeIgnorableWhitespace(false);
	

	try {
        parser->parse(xmlSkillsFilePath);
		xercesc::DOMDocument * document = parser->getDocument();

		retrieveAndCachePhases(document->getChildNodes()->item(1)->getChildNodes()->item(2)->getChildNodes());
		retrieveLowSkills(document->getChildNodes()->item(1)->getChildNodes()->item(1));
		retrieveHighSkills(document->getChildNodes()->item(1)->getChildNodes()->item(0));

    }
    catch (const XMLException& toCatch) {
        char* message = XMLString::transcode(toCatch.getMessage());
        cout << "Exception message is: \n"
             << message << "\n";
        XMLString::release(&message);
        return -1;
    }
    catch (const DOMException& toCatch) {
        char* message = XMLString::transcode(toCatch.getMessage());
        cout << "Exception message is: \n"
             << message << "\n";
        XMLString::release(&message);
        return -1;
    }
	catch (const SAXParseException & toCatch) {
		char* message = XMLString::transcode(toCatch.getMessage());
        cout << "Exception message is: \n"
			<< message << "\n";
        XMLString::release(&message);
        return -1;
    }
	
	delete errorHandler;
	delete parser;
	return 0;
}

void XMLSkillsFactory::retrieveHighSkills(const DOMNode * highSkillsNode) {
	DOMNodeList * children = highSkillsNode->getChildNodes();
	int count = children->getLength();

	DOMElement * highSkillElement;
	
	for (int i = 0; i < count; i++) {
		if( DOMNode::ELEMENT_NODE == children->item(i)->getNodeType() ){
			highSkillElement = dynamic_cast< DOMElement* >( children->item(i) );
			//TODO: barti: get class instance from map upon "name" attribute (E.G - WalkHighSkill)
			string skillName = getAttribute(highSkillElement, "name");
			HighSkill *highSkill = getHighSkillInstance(skillName);
//			HighSkill *highSkill = new WalkHighSkill(transcode(highSkillElement->getAttribute(XMLString::transcode("name"))));

			retrievAndAppendLowSkills(highSkillElement->getChildNodes(), *highSkill);

			highSkills.push_back(highSkill);
		}
	}
}

void XMLSkillsFactory::retrievAndAppendLowSkills(const DOMNodeList * lowSkillsNodes, HighSkill & highSkill) {
	int count = lowSkillsNodes->getLength();
	DOMElement * useSkillElement;
	for (int i = 0; i < count; i++) {
		if( DOMNode::ELEMENT_NODE == lowSkillsNodes->item(i)->getNodeType() ){
			useSkillElement = dynamic_cast< DOMElement* >( lowSkillsNodes->item(i) );
			string key = getAttribute(useSkillElement, "skill");

			if (lowSkills.count(key) > 0) {
				highSkill.addLowSkill(lowSkills.find(key)->second);
			}
		}
	}
}

void XMLSkillsFactory::retrieveLowSkills(const DOMNode * lowSkillsNode) {
	DOMNodeList * children = lowSkillsNode->getChildNodes();
	int count = children->getLength();
	
	DOMElement * lowSkillElement;
	DOMElement * initialPhaseElement;
	DOMNodeList * lowSkillChildren;

	for (int i = 0; i < count; i++) {
		if( DOMNode::ELEMENT_NODE == children->item(i)->getNodeType() ){
			lowSkillElement = dynamic_cast< DOMElement* >( children->item(i) );
			lowSkillChildren = lowSkillElement->getChildNodes();

			string lowSkillName = getAttribute(lowSkillElement, "name");

			initialPhaseElement = getElementFromList(lowSkillChildren, "initial_phase");
			string initialPhase = getAttribute(initialPhaseElement, "name");
			
			LowSkill *lowSkill =  new LowSkill(lowSkillName, initialPhase);
			
			lowSkills.insert(map<string, LowSkill*>::value_type (lowSkill->getName(), lowSkill));
		}
	}
}


void XMLSkillsFactory::retrieveAndCachePhases(const DOMNodeList * phases) {
	int count = phases->getLength();
	
	DOMElement * phaseElement;
	DOMNodeList * phaseNodeChildren;

	for (int i = 0; i < count; i++) {
		if( DOMNode::ELEMENT_NODE == phases->item(i)->getNodeType() ){
			phaseElement = dynamic_cast< DOMElement* >( phases->item(i) );
			phaseNodeChildren = phaseElement->getChildNodes();

			string name = getAttribute(phaseElement, "name");

			int speedConstant = getSingleElementNumberValue(phaseNodeChildren, "speed_constant");

			PhaseData phaseData = PhaseData(name, speedConstant);

			retrieveAndAppendEfectors(getElementFromList(phaseNodeChildren, "efectors"), phaseData);
			

			phaseData.setNext(getAttribute(phaseElement, "next"));

			string final = getAttribute(phaseElement, "isFinal");

			if (final == "true") {
				phaseData.setFinal(true);
			} else {
				phaseData.setFinal(false);
			}

			phaseData.setFinalizationPhase(getSingleElementValue(phaseNodeChildren, "finalization_phase"));

			phaseData.setRescueMovement(getSingleElementValue(phaseNodeChildren, "rescue_movement"));

			phaseCache->put(name, phaseData);

		}
	}
}

void XMLSkillsFactory::retrieveAndAppendEfectors(DOMNode * efectorsNode, PhaseData & phaseData) {
	int count = efectorsNode->getChildNodes()->getLength();

	for (int i = 0; i < count; i++) {
		retrieveAndSetEfector(efectorsNode->getChildNodes()->item(i), phaseData);
	}
}

void XMLSkillsFactory::retrieveAndSetEfector(DOMNode * efectorNode, PhaseData & phaseData) {
	DOMElement * efectorElement;
	if( DOMNode::ELEMENT_NODE == efectorNode->getNodeType() ){
		efectorElement = dynamic_cast< DOMElement* >( efectorNode );
		
		string efectorName = getAttribute(efectorElement, "name");

		int efectorStart = getSingleElementNumberValue(efectorElement->getChildNodes(), "start");
		int efectorEnd = getSingleElementNumberValue(efectorElement->getChildNodes(), "end");

		EfectorData efectorData = EfectorData(efectorName, efectorStart, efectorEnd);
		phaseData.addEfectorData(efectorData);
	} 
}

int XMLSkillsFactory::getSingleElementNumberValue(const DOMNodeList * list, const string & elementName) {
	DOMElement * element = getElementFromList(list, elementName);
	return XMLString::parseInt(element->getChildNodes()->item(0)->getNodeValue());
}

string XMLSkillsFactory::getSingleElementValue(const DOMNodeList * list, const string & elementName) {
	DOMElement * element = getElementFromList(list, elementName);
	return transcode(element->getChildNodes()->item(0)->getNodeValue());
}

DOMElement * XMLSkillsFactory::getElementFromList(const DOMNodeList * list, const string & elementName) {
	int count = list->getLength();

	for (int i = 0; i < count; i++) {
		if (transcode(list->item(i)->getNodeName()) == elementName) {
			if (DOMNode::ELEMENT_NODE == list->item(i)->getNodeType()) {
				return dynamic_cast<DOMElement *> (list->item(i));
			}
		}
	}
	return 0;
}

string XMLSkillsFactory::getAttribute(const DOMElement * attrElement, const string & attrName) {
	return transcode(attrElement->getAttribute(XMLString::transcode(attrName.c_str())));
}


string XMLSkillsFactory::transcode(const XMLCh * xmlChars)
{
	return XMLString::transcode(xmlChars);
}

XMLSkillsFactory::~XMLSkillsFactory() {

}
