#pragma once

#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/dom/DOM.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/sax/HandlerBase.hpp>
#include "../skillsFactory/SkillsFactory.h"
#include "../lowSkills/LowSkill.h"
#include "../lowSkills/PhaseData.h"
#include "../lowSkills/PhaseCache.h"
#include "../highSkills/GoToPointHighSkill.h"
#include "../highSkills/StandUpHighSkill.h"
#include "../highSkills/KickHighSkill.h"
#include "../highSkills/WalkBackHighSkill.h"
#include "../lowSkills/EfectorData.h"
#include "../highSkills/TurnToBallHighSkill.h"
#include <string>
#include <vector>
#include <map>
#include <ctime>
#include <sys/stat.h>

using namespace std;
using namespace xercesc;

class LowSkill;
class GoToPointHighSkill;
class HighSkill;
class XMLSkillsFactory : public SkillsFactory
{
private:
	char *  xmlSkillsFilePath;

	time_t lastTimeModified;

	PhaseCache * phaseCache;

	string transcode(const XMLCh * xmlChars);

	map<string, LowSkill*> lowSkills;

	HighSkill *getHighSkillInstance(string name);
	void retrieveHighSkills(const DOMNode * highSkillsNode);
	void retrievAndAppendLowSkills(const DOMNodeList * lowSkillsNodes, HighSkill & highSkill);
	void retrieveLowSkills(const DOMNode * lowSkillsNode);
	void retrieveAndCachePhases(const DOMNodeList * phases);
	void retrieveAndAppendEfectors(DOMNode * efectorsNode, PhaseData & phaseData);
	void retrieveAndSetEfector(DOMNode * efectorNode, PhaseData & phaseData);

	string getAttribute(const DOMElement * attrElement, const string & attrName);
	string getSingleElementValue(const DOMNodeList * list, const string & elementName);
	int getSingleElementNumberValue(const DOMNodeList * list, const string & elementName);
	DOMElement * getElementFromList(const DOMNodeList * list, const string & elementName);


	bool parsingNeeded();
	void prepare();
	int parse();
public:
	XMLSkillsFactory();

	vector<HighSkill*> * getAllHighSkills();

	~XMLSkillsFactory();
};