/*
 * TCPSocket.cpp
 */

#include "TCPSocket.h"
#include <iostream>

using namespace std;

TCPSocket::TCPSocket()
{
#ifdef WIN32
	WSAData wsaData;
	if(WSAStartup(MAKEWORD(2, 2), &wsaData) != -1)
	{
		cout << "Agent spustil WSA." << endl;
	}
	else
	{
		cerr << "Agent nedokazal spustit WSA." << endl;
	}
#endif
	fileDescriptor = ::socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (fileDescriptor != -1)
	{
		cout << "Agent vytvoril soket." << endl;
	}
	else
	{
		cerr << "Agent nedokazal vytvorit soket." << endl;
	}
}

bool TCPSocket::connect(struct sockaddr *server)
{
	if (::connect(fileDescriptor, server, sizeof(*server)) != -1)
	{
		cout << "Agent sa pripojil na server." << endl;
		return true;
	}
	else
	{
		cerr << "Agent sa nedokazal pripojit na server." << endl;
		return false;
	}
}

bool TCPSocket::send(const char *message)
{
	int messageLength = (int) strlen(message);
	if (messageLength > 0)
	{
		int messageLength2 = htonl(messageLength);
		char *message2 = new char[sizeof(messageLength2) + messageLength];
		memcpy(message2, &messageLength2, sizeof(messageLength2));
		memcpy(message2 + sizeof(messageLength2), message, messageLength);
		int bytes = ::send(fileDescriptor, message2, sizeof(messageLength2) + messageLength, 0);
		delete[] message2;
		if (bytes != -1)
		{
			return true;
		}
		else
		{
			cerr << "Agent nedokazal poslat data." << endl;
			return false;
		}
	}
	else
	{
		return true;
	}
}

bool TCPSocket::select(int seconds, int microseconds)
{
	fd_set fileDescriptorSet;
	FD_ZERO(&fileDescriptorSet);
	FD_SET(fileDescriptor, &fileDescriptorSet);
	struct timeval timeout;
	timeout.tv_sec = seconds;
	timeout.tv_usec = microseconds;
	return ::select((int) fileDescriptor + 1, &fileDescriptorSet, NULL, NULL, &timeout) > 0;
}

bool TCPSocket::receive(char *message, int messageSize)
{
	int messageLength = 0;
	int messageLength2;
	char *message2 = new char[sizeof(messageLength2) + messageSize - 1];
	int flags = MSG_PEEK;
	bool run = true;
	do
	{
		int bytes = ::recv(fileDescriptor, message2, sizeof(messageLength2) + messageLength, flags);
		if (bytes == sizeof(messageLength2))
		{
			memcpy(&messageLength2, message2, sizeof(messageLength2));
			messageLength = ntohl(messageLength2);
			if (messageSize - 1 < messageLength)
			{
				cerr << "Agent nedokazal prijat data, pretoze su prilis dlhe." << endl;
				return false;
			}
		}
		else if (bytes == (int) sizeof(messageLength2) + messageLength)
		{
			if (flags == MSG_PEEK)
			{
				flags = 0;
			}
			else
			{
				run = false;
			}
		}
		else if (bytes == -1)
		{
			cerr << "Agent nedokazal prijat data." << endl;
			return false;
		}
		else if (bytes == 0)
		{
			cout << "Agent neprijal ziadne data." << endl;
			return false;
		}
	}
	while (run);
	memcpy(message, message2 + sizeof(messageLength2), messageLength);
	message[messageLength] = '\0';
	delete[] message2;
	return true;
}

TCPSocket::~TCPSocket()
{
#ifdef WIN32
	if (::closesocket(fileDescriptor) != -1)
#else
	if (::close(fileDescriptor) != -1)
#endif
	{
		cout << "Agent zatvoril soket." << endl;
	}
	else
	{
		cerr << "Agent nedokazal zatvorit soket." << endl;
	}
#ifdef WIN32
	if (WSACleanup() != -1)
	{
		cout << "Agent vycistil WSA." << endl;
	}
	else
	{
		cerr << "Agent nedokazal vycistit WSA." << endl;
	}
#endif
}
