/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  
	 created: Tue Nov 11 22:30:20 2008
	filename: ServerMsgParser.cpp
	  author: Lowell Thomas
	          lowell@coasttocoastresearch.com
	          http://www.coasttocoastresearch.com

	 purpose: Implementation of parser class ApgServerMsgParser.

*********************************************************************/
//{{Include.pre
#include "../../../include/Apg.h"
#include "ServerMsgParser.h"

//}}Include.pre


//#define MAX_BUFFER_SIZE 1024

//{{Include.post
char *psr_msg_string;		//msg from server
char parse_buffer[MAX_BUFFER_SIZE];	//temp buffer
ServerMsgParserData *pStoredData;
int curr_obj_id;
int frp_curr_foot;		// 0-left, 1-right
Player tempPlayer;
HearPerceptor tempHear;

//NAO
TJointIDMap::iterator idIter;
JointID jid;

//}}Include.post

//  GENERAL INFORMATION:
//       static data = 7016 (bytes)
//                   = 6.85E+000 (KB)
//                   = 6.69E-003 (MB)
//   character count = 123 (grammar alphabet character set)
// unused characters = 55 (grammar alphabet character set)
//        rule count = 70
//      opcode count = 336
//         ALT count = 13(47) (alternations)
//         CAT count = 17(157) (concatenations)
//         RNM count = 153
//         REP count = 62
//         PRD count = 0
//         TRG count = 4
//         TBS count = 0
//         TLS count = 87
//      string table = 732 (bytes)
//           opcodes = 4524 (bytes)
//         rule list = 1400 (bytes)
//              PPTs = <not used>

// CALL BACK FUNCTION POINTERS
static ulong g_ulCallbacks = 32;
static PFN_CALLBACK g_pfnCallbacks[32] = 
{
  &ApgServerMsgParser::pfn_Time,
  &ApgServerMsgParser::pfn_UnumNr,
  &ApgServerMsgParser::pfn_TeamLeft,
  &ApgServerMsgParser::pfn_TeamRight,
  &ApgServerMsgParser::pfn_GSTime,
  &ApgServerMsgParser::pfn_PlayMode,
  &ApgServerMsgParser::pfn_GYRx,
  &ApgServerMsgParser::pfn_GYRy,
  &ApgServerMsgParser::pfn_GYRz,
  &ApgServerMsgParser::pfn_ObjectName,
  &ApgServerMsgParser::pfn_SEETeamName,
  &ApgServerMsgParser::pfn_SEEPlayerID,
  &ApgServerMsgParser::pfn_SEEx,
  &ApgServerMsgParser::pfn_SEEy,
  &ApgServerMsgParser::pfn_SEEz,
  &ApgServerMsgParser::pfn_SEEPx,
  &ApgServerMsgParser::pfn_SEEPy,
  &ApgServerMsgParser::pfn_SEEPz,
  &ApgServerMsgParser::pfn_HJName,
  &ApgServerMsgParser::pfn_HJax,
  &ApgServerMsgParser::pfn_FNameLF,
  &ApgServerMsgParser::pfn_FNameRF,
  &ApgServerMsgParser::pfn_FRPCx,
  &ApgServerMsgParser::pfn_FRPCy,
  &ApgServerMsgParser::pfn_FRPCz,
  &ApgServerMsgParser::pfn_FRPFx,
  &ApgServerMsgParser::pfn_FRPFy,
  &ApgServerMsgParser::pfn_FRPFz,
  &ApgServerMsgParser::pfn_HEARtime,
  &ApgServerMsgParser::pfn_HEARxy,
  &ApgServerMsgParser::pfn_HEARself,
  &ApgServerMsgParser::pfn_HEARmessage,
};

// EXECUTABLE DATA
static ulong g_ulaExeData[1684] = 
{
  0x00001A50,0x7D9064E3,0x00000000,0x00000046,0x00000150,0x0000007B,0x00000000,0x0000007B,
  0x0000007A,0x00000000,0x00000000,0x00000050,0x00000050,0x000002DC,0x0000032C,0x000011AC,
  0x000014D8,0x00000578,0x00000000,0x00000000,0x7373654D,0x00656761,0x4567734D,0x4D006D6C,
  0x69546773,0x2800656D,0x776F6E28,0x69742929,0x7453656D,0x69740072,0x6954656D,0x4D00656D,
  0x61476773,0x7453656D,0x00657461,0x53472928,0x4567734D,0x4700626D,0x67734D53,0x656D6954,
  0x29742800,0x734D5347,0x4D6C5067,0x0065646F,0x296D7028,0x734D5347,0x756E5567,0x7528006D,
  0x296D756E,0x734D5347,0x61655467,0x7428006D,0x296D6165,0x6D756E55,0x5400724E,0x4C6D6165,
  0x00746665,0x7466656C,0x6D616554,0x68676952,0x69720074,0x47746867,0x72745353,0x47736700,
  0x6D695453,0x6C500065,0x6F4D7961,0x4D006564,0x79476773,0x61526F72,0x28006574,0x6F746E28,
  0x296F7372,0x29747228,0x72794729,0x00727453,0x47727967,0x00785259,0x79525947,0x52594700,
  0x734D007A,0x65655367,0x53292800,0x624F6565,0x7463656A,0x65655300,0x4F656E4F,0x63656A62,
  0x28280074,0x296C6F70,0x65655329,0x50656E4F,0x6579616C,0x28280072,0x6D616574,0x64692829,
  0x6F702829,0x5329296C,0x74536565,0x65730072,0x74535065,0x4F700072,0x63656A62,0x6D614E74,
  0x45530065,0x61655445,0x6D614E6D,0x45530065,0x616C5045,0x49726579,0x45530044,0x53007845,
  0x00794545,0x7A454553,0x45455300,0x53007850,0x79504545,0x45455300,0x4D007A50,0x69486773,
  0x0065676E,0x296E2828,0x29786128,0x534A4829,0x68007274,0x4E4A486A,0x00656D61,0x78614A48,
  0x67734D00,0x63726F46,0x73655265,0x61747369,0x0065636E,0x296E2828,0x28296328,0x46292966,
  0x74535052,0x72660072,0x614E4670,0x4600656D,0x656D614E,0x6C00464C,0x614E4666,0x4652656D,
  0x46667200,0x78435052,0x50524600,0x46007943,0x7A435052,0x50524600,0x46007846,0x79465052,
  0x50524600,0x4D007A46,0x65486773,0x28007261,0x61654829,0x52545372,0x61656800,0x41454872,
  0x6D697452,0x45480065,0x69645241,0x74636572,0x006E6F69,0x52414548,0x48007978,0x73524145,
  0x00666C65,0x666C6573,0x52414548,0x7373656D,0x00656761,0x5567734D,0x6F6E6B6E,0x29280077,
  0x5467734D,0x006C6961,0x41005053,0x4148504C,0x47494400,0x72005449,0x4E6C6165,0x2E2D0072,
  0x68706C61,0x6D756E61,0x69727473,0x6200676E,0x6B636172,0x732D7465,0x6E697274,0x29280067,
  0x69727473,0x772D676E,0x6F687469,0x622D7475,0x2E5F0072,0x69727473,0x2800676E,0x692E5F29,
  0x746E6564,0x65696669,0x005F0072,0x00000003,0x0000001C,0x00000000,0xFFFFFFFF,0x00000000,
  0x0000001C,0x00000001,0x00000001,0x00000090,0x00000000,0x00000030,0x00000002,0x00000000,
  0x0000003C,0x00000005,0x00000000,0x00000048,0x00000034,0x00000000,0x00000054,0x00000011,
  0x00000000,0x00000060,0x00000025,0x00000000,0x0000006C,0x00000016,0x00000000,0x00000078,
  0x00000029,0x00000000,0x00000084,0x0000003B,0x00000000,0x00000090,0x0000003C,0x00000002,
  0x00000128,0x00000007,0x000000A8,0x00000017,0x00000001,0x00000000,0x000000B4,0x00000003,
  0x00000003,0x000000D0,0x00000000,0xFFFFFFFF,0x00000000,0x000000D0,0x0000003D,0x00000007,
  0x000000E0,0x00000018,0x00000001,0x00000007,0x000000F0,0x00000019,0x00000003,0x00000003,
  0x0000010C,0x00000000,0xFFFFFFFF,0x00000000,0x0000010C,0x0000003D,0x00000000,0x00000118,
  0x00000004,0x00000007,0x00000128,0x0000001C,0x00000002,0x00000007,0x00000138,0x00000026,
  0x00000004,0x00000000,0x00000144,0x00000040,0x00000002,0x000001D4,0x00000007,0x0000015C,
  0x0000003C,0x00000001,0x00000000,0x00000168,0x0000000E,0x00000003,0x00000184,0x00000000,
  0xFFFFFFFF,0x00000000,0x00000184,0x0000003D,0x00000003,0x000001C4,0x00000000,0xFFFFFFFF,
  0x00000002,0x000001C4,0x00000000,0x000001A8,0x00000006,0x00000003,0x000001C4,0x00000000,
  0xFFFFFFFF,0x00000000,0x000001C4,0x0000003D,0x00000007,0x000001D4,0x0000003D,0x00000001,
  0x00000001,0x0000020C,0x00000000,0x000001E8,0x00000007,0x00000000,0x000001F4,0x00000008,
  0x00000000,0x00000200,0x00000009,0x00000000,0x0000020C,0x0000000A,0x00000002,0x0000026C,
  0x00000007,0x00000224,0x00000051,0x00000001,0x00000007,0x00000234,0x00000052,0x00000001,
  0x00000003,0x00000250,0x00000000,0xFFFFFFFF,0x00000000,0x00000250,0x0000003D,0x00000000,
  0x0000025C,0x0000000F,0x00000007,0x0000026C,0x00000053,0x00000001,0x00000002,0x000002CC,
  0x00000007,0x00000284,0x00000060,0x00000001,0x00000007,0x00000294,0x00000061,0x00000002,
  0x00000003,0x000002B0,0x00000000,0xFFFFFFFF,0x00000000,0x000002B0,0x0000003D,0x00000000,
  0x000002BC,0x00000010,0x00000007,0x000002CC,0x00000063,0x00000001,0x00000002,0x0000032C,
  0x00000007,0x000002E4,0x0000006E,0x00000001,0x00000007,0x000002F4,0x0000006F,0x00000004,
  0x00000003,0x00000310,0x00000000,0xFFFFFFFF,0x00000000,0x00000310,0x0000003D,0x00000000,
  0x0000031C,0x0000000B,0x00000007,0x0000032C,0x00000073,0x00000001,0x00000002,0x000003A0,
  0x00000007,0x00000344,0x0000007E,0x00000001,0x00000007,0x00000354,0x0000007F,0x00000004,
  0x00000003,0x00000370,0x00000000,0xFFFFFFFF,0x00000000,0x00000370,0x0000003D,0x00000001,
  0x00000390,0x00000000,0x00000384,0x0000000C,0x00000000,0x00000390,0x0000000D,0x00000007,
  0x000003A0,0x00000083,0x00000001,0x00000003,0x000003BC,0x00000001,0xFFFFFFFF,0x00000000,
  0x000003BC,0x0000003F,0x00000007,0x000003CC,0x00000094,0x00000004,0x00000007,0x000003DC,
  0x000000A2,0x00000005,0x00000007,0x000003EC,0x000000AD,0x00000002,0x00000000,0x000003F8,
  0x00000040,0x00000000,0x00000404,0x00000045,0x00000002,0x00000564,0x00000007,0x0000041C,
  0x000000CB,0x00000001,0x00000000,0x00000428,0x00000012,0x00000003,0x00000444,0x00000000,
  0xFFFFFFFF,0x00000000,0x00000444,0x0000003D,0x00000007,0x00000454,0x000000CC,0x00000001,
  0x00000007,0x00000464,0x000000CD,0x00000001,0x00000003,0x00000480,0x00000000,0xFFFFFFFF,
  0x00000000,0x00000480,0x0000003D,0x00000007,0x00000490,0x000000CE,0x00000005,0x00000007,
  0x000004A0,0x000000D3,0x00000001,0x00000003,0x000004BC,0x00000000,0xFFFFFFFF,0x00000000,
  0x000004BC,0x0000003D,0x00000007,0x000004CC,0x000000D4,0x00000001,0x00000007,0x000004DC,
  0x000000D5,0x00000002,0x00000003,0x000004F8,0x00000000,0xFFFFFFFF,0x00000000,0x000004F8,
  0x0000003D,0x00000000,0x00000504,0x00000013,0x00000003,0x00000520,0x00000000,0xFFFFFFFF,
  0x00000000,0x00000520,0x0000003D,0x00000000,0x0000052C,0x00000014,0x00000003,0x00000548,
  0x00000000,0xFFFFFFFF,0x00000000,0x00000548,0x0000003D,0x00000000,0x00000554,0x00000015,
  0x00000007,0x00000564,0x000000D7,0x00000002,0x00000007,0x00000574,0x000000E0,0x00000003,
  0x00000000,0x00000580,0x00000040,0x00000000,0x0000058C,0x00000040,0x00000000,0x00000598,
  0x00000040,0x00000002,0x00000604,0x00000007,0x000005B0,0x000000F9,0x00000001,0x00000000,
  0x000005BC,0x0000001A,0x00000003,0x000005D8,0x00000000,0xFFFFFFFF,0x00000000,0x000005D8,
  0x0000003D,0x00000003,0x000005F4,0x00000000,0xFFFFFFFF,0x00000000,0x000005F4,0x00000017,
  0x00000007,0x00000604,0x000000FA,0x00000001,0x00000001,0x00000624,0x00000000,0x00000618,
  0x00000018,0x00000000,0x00000624,0x00000019,0x00000002,0x00000728,0x00000007,0x0000063C,
  0x00000112,0x00000001,0x00000000,0x00000648,0x0000001C,0x00000003,0x00000664,0x00000000,
  0xFFFFFFFF,0x00000000,0x00000664,0x0000003D,0x00000007,0x00000674,0x00000113,0x00000001,
  0x00000007,0x00000684,0x00000114,0x00000003,0x00000003,0x000006A0,0x00000000,0xFFFFFFFF,
  0x00000000,0x000006A0,0x0000003D,0x00000000,0x000006AC,0x0000001F,0x00000003,0x000006C8,
  0x00000000,0xFFFFFFFF,0x00000000,0x000006C8,0x0000003D,0x00000000,0x000006D4,0x00000020,
  0x00000003,0x000006F0,0x00000000,0xFFFFFFFF,0x00000000,0x000006F0,0x0000003D,0x00000000,
  0x000006FC,0x00000021,0x00000007,0x0000070C,0x00000117,0x00000002,0x00000003,0x00000728,
  0x00000000,0xFFFFFFFF,0x00000000,0x00000728,0x0000003D,0x00000002,0x00000914,0x00000007,
  0x00000740,0x00000126,0x00000001,0x00000000,0x0000074C,0x0000001B,0x00000003,0x00000768,
  0x00000000,0xFFFFFFFF,0x00000000,0x00000768,0x0000003D,0x00000007,0x00000778,0x00000127,
  0x00000001,0x00000007,0x00000788,0x00000128,0x00000004,0x00000003,0x000007A4,0x00000000,
  0xFFFFFFFF,0x00000000,0x000007A4,0x0000003D,0x00000000,0x000007B0,0x0000001D,0x00000007,
  0x000007C0,0x0000012C,0x00000001,0x00000003,0x000007DC,0x00000000,0xFFFFFFFF,0x00000000,
  0x000007DC,0x0000003D,0x00000007,0x000007EC,0x0000012D,0x00000001,0x00000007,0x000007FC,
  0x0000012E,0x00000002,0x00000003,0x00000818,0x00000000,0xFFFFFFFF,0x00000000,0x00000818,
  0x0000003D,0x00000000,0x00000824,0x0000001E,0x00000007,0x00000834,0x00000130,0x00000001,
  0x00000003,0x00000850,0x00000000,0xFFFFFFFF,0x00000000,0x00000850,0x0000003D,0x00000007,
  0x00000860,0x00000131,0x00000001,0x00000007,0x00000870,0x00000132,0x00000003,0x00000003,
  0x0000088C,0x00000000,0xFFFFFFFF,0x00000000,0x0000088C,0x0000003D,0x00000000,0x00000898,
  0x00000022,0x00000003,0x000008B4,0x00000000,0xFFFFFFFF,0x00000000,0x000008B4,0x0000003D,
  0x00000000,0x000008C0,0x00000023,0x00000003,0x000008DC,0x00000000,0xFFFFFFFF,0x00000000,
  0x000008DC,0x0000003D,0x00000000,0x000008E8,0x00000024,0x00000007,0x000008F8,0x00000135,
  0x00000002,0x00000003,0x00000914,0x00000000,0xFFFFFFFF,0x00000000,0x00000914,0x0000003D,
  0x00000007,0x00000924,0x0000013E,0x00000003,0x00000007,0x00000934,0x00000146,0x00000001,
  0x00000000,0x00000940,0x00000045,0x00000000,0x0000094C,0x00000045,0x00000003,0x00000968,
  0x00000001,0xFFFFFFFF,0x00000000,0x00000968,0x0000003F,0x00000000,0x00000974,0x00000040,
  0x00000000,0x00000980,0x00000040,0x00000000,0x0000098C,0x00000040,0x00000000,0x00000998,
  0x00000040,0x00000000,0x000009A4,0x00000040,0x00000000,0x000009B0,0x00000040,0x00000002,
  0x00000ABC,0x00000007,0x000009C8,0x00000194,0x00000001,0x00000000,0x000009D4,0x00000026,
  0x00000003,0x000009F0,0x00000000,0xFFFFFFFF,0x00000000,0x000009F0,0x0000003D,0x00000007,
  0x00000A00,0x00000195,0x00000001,0x00000007,0x00000A10,0x00000196,0x00000001,0x00000003,
  0x00000A2C,0x00000000,0xFFFFFFFF,0x00000000,0x00000A2C,0x0000003D,0x00000000,0x00000A38,
  0x00000027,0x00000007,0x00000A48,0x00000197,0x00000001,0x00000003,0x00000A64,0x00000000,
  0xFFFFFFFF,0x00000000,0x00000A64,0x0000003D,0x00000007,0x00000A74,0x00000198,0x00000001,
  0x00000007,0x00000A84,0x00000199,0x00000002,0x00000003,0x00000AA0,0x00000000,0xFFFFFFFF,
  0x00000000,0x00000AA0,0x0000003D,0x00000000,0x00000AAC,0x00000028,0x00000007,0x00000ABC,
  0x0000019B,0x00000002,0x00000007,0x00000ACC,0x000001A3,0x00000002,0x00000000,0x00000AD8,
  0x00000045,0x00000000,0x00000AE4,0x00000040,0x00000002,0x00000D04,0x00000007,0x00000AFC,
  0x000001C4,0x00000001,0x00000000,0x00000B08,0x0000002A,0x00000003,0x00000B24,0x00000000,
  0xFFFFFFFF,0x00000000,0x00000B24,0x0000003D,0x00000007,0x00000B34,0x000001C5,0x00000001,
  0x00000007,0x00000B44,0x000001C6,0x00000001,0x00000003,0x00000B60,0x00000000,0xFFFFFFFF,
  0x00000000,0x00000B60,0x0000003D,0x00000000,0x00000B6C,0x0000002B,0x00000007,0x00000B7C,
  0x000001C7,0x00000001,0x00000003,0x00000B98,0x00000000,0xFFFFFFFF,0x00000000,0x00000B98,
  0x0000003D,0x00000007,0x00000BA8,0x000001C8,0x00000001,0x00000007,0x00000BB8,0x000001C9,
  0x00000001,0x00000003,0x00000BD4,0x00000000,0xFFFFFFFF,0x00000000,0x00000BD4,0x0000003D,
  0x00000000,0x00000BE0,0x0000002E,0x00000003,0x00000BFC,0x00000000,0xFFFFFFFF,0x00000000,
  0x00000BFC,0x0000003D,0x00000000,0x00000C08,0x0000002F,0x00000003,0x00000C24,0x00000000,
  0xFFFFFFFF,0x00000000,0x00000C24,0x0000003D,0x00000000,0x00000C30,0x00000030,0x00000007,
  0x00000C40,0x000001CA,0x00000001,0x00000003,0x00000C5C,0x00000000,0xFFFFFFFF,0x00000000,
  0x00000C5C,0x0000003D,0x00000007,0x00000C6C,0x000001CB,0x00000001,0x00000007,0x00000C7C,
  0x000001CC,0x00000001,0x00000003,0x00000C98,0x00000000,0xFFFFFFFF,0x00000000,0x00000C98,
  0x0000003D,0x00000000,0x00000CA4,0x00000031,0x00000003,0x00000CC0,0x00000000,0xFFFFFFFF,
  0x00000000,0x00000CC0,0x0000003D,0x00000000,0x00000CCC,0x00000032,0x00000003,0x00000CE8,
  0x00000000,0xFFFFFFFF,0x00000000,0x00000CE8,0x0000003D,0x00000000,0x00000CF4,0x00000033,
  0x00000007,0x00000D04,0x000001CD,0x00000002,0x00000007,0x00000D14,0x000001D6,0x00000003,
  0x00000001,0x00000D34,0x00000000,0x00000D28,0x0000002C,0x00000000,0x00000D34,0x0000002D,
  0x00000007,0x00000D44,0x000001E7,0x00000002,0x00000007,0x00000D54,0x000001F1,0x00000002,
  0x00000000,0x00000D60,0x00000040,0x00000000,0x00000D6C,0x00000040,0x00000000,0x00000D78,
  0x00000040,0x00000000,0x00000D84,0x00000040,0x00000000,0x00000D90,0x00000040,0x00000000,
  0x00000D9C,0x00000040,0x00000002,0x00000E58,0x00000007,0x00000DB4,0x0000021F,0x00000001,
  0x00000000,0x00000DC0,0x00000035,0x00000003,0x00000DDC,0x00000000,0xFFFFFFFF,0x00000000,
  0x00000DDC,0x0000003D,0x00000000,0x00000DE8,0x00000036,0x00000003,0x00000E04,0x00000000,
  0xFFFFFFFF,0x00000000,0x00000E04,0x0000003D,0x00000003,0x00000E20,0x00000000,0xFFFFFFFF,
  0x00000000,0x00000E20,0x00000037,0x00000003,0x00000E3C,0x00000000,0xFFFFFFFF,0x00000000,
  0x00000E3C,0x0000003D,0x00000000,0x00000E48,0x0000003A,0x00000007,0x00000E58,0x00000220,
  0x00000001,0x00000007,0x00000E68,0x00000229,0x00000004,0x00000000,0x00000E74,0x00000040,
  0x00000001,0x00000E94,0x00000000,0x00000E88,0x00000038,0x00000000,0x00000E94,0x00000039,
  0x00000000,0x00000EA0,0x00000040,0x00000007,0x00000EB0,0x00000254,0x00000004,0x00000000,
  0x00000EBC,0x00000045,0x00000002,0x00000F30,0x00000007,0x00000ED4,0x0000026E,0x00000001,
  0x00000003,0x00000F20,0x00000000,0xFFFFFFFF,0x00000001,0x00000F20,0x00000000,0x00000EF8,
  0x00000043,0x00000000,0x00000F04,0x00000042,0x00000003,0x00000F20,0x00000000,0xFFFFFFFF,
  0x00000000,0x00000F20,0x0000003D,0x00000007,0x00000F30,0x0000026F,0x00000001,0x00000000,
  0x00000F3C,0x00000044,0x00000005,0x00000F4C,0x00000020,0x00000020,0x00000001,0x00000F74,
  0x00000005,0x00000F64,0x00000041,0x0000005A,0x00000005,0x00000F74,0x00000061,0x0000007A,
  0x00000005,0x00000F84,0x00000030,0x00000039,0x00000002,0x00000FF4,0x00000003,0x00000FAC,
  0x00000000,0x00000001,0x00000007,0x00000FAC,0x0000028E,0x00000001,0x00000003,0x00000FC8,
  0x00000001,0xFFFFFFFF,0x00000000,0x00000FC8,0x0000003F,0x00000007,0x00000FD8,0x0000028F,
  0x00000001,0x00000003,0x00000FF4,0x00000001,0xFFFFFFFF,0x00000000,0x00000FF4,0x0000003F,
  0x00000003,0x00001030,0x00000000,0xFFFFFFFF,0x00000001,0x00001030,0x00000000,0x00001018,
  0x0000003E,0x00000000,0x00001024,0x0000003F,0x00000000,0x00001030,0x0000003D,0x00000002,
  0x00001094,0x00000007,0x00001048,0x000002AE,0x00000001,0x00000003,0x00001084,0x00000000,
  0xFFFFFFFF,0x00000001,0x00001084,0x00000000,0x0000106C,0x00000041,0x00000000,0x00001078,
  0x00000042,0x00000000,0x00001084,0x0000003D,0x00000007,0x00001094,0x000002AF,0x00000001,
  0x00000003,0x000010F0,0x00000000,0xFFFFFFFF,0x00000001,0x000010F0,0x00000000,0x000010B8,
  0x0000003E,0x00000000,0x000010C4,0x0000003F,0x00000000,0x000010D0,0x0000003D,0x00000007,
  0x000010E0,0x000002C2,0x00000001,0x00000007,0x000010F0,0x000002C3,0x00000001,0x00000003,
  0x0000116C,0x00000000,0xFFFFFFFF,0x00000001,0x0000116C,0x00000000,0x00001114,0x0000003E,
  0x00000000,0x00001120,0x0000003F,0x00000000,0x0000112C,0x0000003D,0x00000007,0x0000113C,
  0x000002CB,0x00000001,0x00000007,0x0000114C,0x000002CC,0x00000001,0x00000007,0x0000115C,
  0x000002CD,0x00000001,0x00000007,0x0000116C,0x000002CE,0x00000001,0x00000003,0x000011AC,
  0x00000001,0xFFFFFFFF,0x00000001,0x000011AC,0x00000000,0x00001190,0x0000003E,0x00000000,
  0x0000119C,0x0000003F,0x00000007,0x000011AC,0x000002DA,0x00000001,0x00000000,0x00000000,
  0x00000000,0x00000000,0xFFFFFFFF,0x00000008,0x00000000,0x0000001C,0x00000000,0xFFFFFFFF,
  0x0000000F,0x00000000,0x00000090,0x00000000,0xFFFFFFFF,0x0000001E,0x00000000,0x00000128,
  0x00000000,0xFFFFFFFF,0x0000002A,0x00000000,0x00000138,0x00000001,0x00000000,0x0000002F,
  0x00000000,0x00000144,0x00000000,0xFFFFFFFF,0x0000003E,0x00000000,0x000001D4,0x00000000,
  0xFFFFFFFF,0x00000047,0x00000000,0x0000020C,0x00000000,0xFFFFFFFF,0x00000054,0x00000000,
  0x0000026C,0x00000000,0xFFFFFFFF,0x00000064,0x00000000,0x000002CC,0x00000000,0xFFFFFFFF,
  0x00000074,0x00000000,0x0000032C,0x00000000,0xFFFFFFFF,0x00000084,0x00000000,0x000003A0,
  0x00000001,0x00000001,0x0000008B,0x00000000,0x000003BC,0x00000001,0x00000002,0x00000098,
  0x00000000,0x000003CC,0x00000001,0x00000003,0x000000A7,0x00000000,0x000003DC,0x00000000,
  0xFFFFFFFF,0x000000AF,0x00000000,0x000003EC,0x00000001,0x00000004,0x000000B6,0x00000000,
  0x000003F8,0x00000001,0x00000005,0x000000BF,0x00000000,0x00000404,0x00000000,0xFFFFFFFF,
  0x000000D9,0x00000000,0x00000564,0x00000000,0xFFFFFFFF,0x000000E3,0x00000000,0x00000574,
  0x00000001,0x00000006,0x000000E8,0x00000000,0x00000580,0x00000001,0x00000007,0x000000ED,
  0x00000000,0x0000058C,0x00000001,0x00000008,0x000000F2,0x00000000,0x00000598,0x00000000,
  0xFFFFFFFF,0x000000FB,0x00000000,0x00000604,0x00000000,0xFFFFFFFF,0x00000105,0x00000000,
  0x00000624,0x00000000,0xFFFFFFFF,0x00000119,0x00000000,0x00000728,0x00000000,0xFFFFFFFF,
  0x00000137,0x00000000,0x00000914,0x00000000,0xFFFFFFFF,0x00000141,0x00000000,0x00000924,
  0x00000000,0xFFFFFFFF,0x00000147,0x00000000,0x00000934,0x00000001,0x00000009,0x00000152,
  0x00000000,0x00000940,0x00000001,0x0000000A,0x0000015E,0x00000000,0x0000094C,0x00000001,
  0x0000000B,0x0000016A,0x00000000,0x00000968,0x00000001,0x0000000C,0x0000016F,0x00000000,
  0x00000974,0x00000001,0x0000000D,0x00000174,0x00000000,0x00000980,0x00000001,0x0000000E,
  0x00000179,0x00000000,0x0000098C,0x00000001,0x0000000F,0x0000017F,0x00000000,0x00000998,
  0x00000001,0x00000010,0x00000185,0x00000000,0x000009A4,0x00000001,0x00000011,0x0000018B,
  0x00000000,0x000009B0,0x00000000,0xFFFFFFFF,0x0000019D,0x00000000,0x00000ABC,0x00000000,
  0xFFFFFFFF,0x000001A5,0x00000000,0x00000ACC,0x00000001,0x00000012,0x000001AC,0x00000000,
  0x00000AD8,0x00000001,0x00000013,0x000001B1,0x00000000,0x00000AE4,0x00000000,0xFFFFFFFF,
  0x000001CF,0x00000000,0x00000D04,0x00000000,0xFFFFFFFF,0x000001D9,0x00000000,0x00000D14,
  0x00000000,0xFFFFFFFF,0x000001DF,0x00000000,0x00000D34,0x00000001,0x00000014,0x000001E9,
  0x00000000,0x00000D44,0x00000001,0x00000015,0x000001F3,0x00000000,0x00000D54,0x00000001,
  0x00000016,0x000001F9,0x00000000,0x00000D60,0x00000001,0x00000017,0x000001FF,0x00000000,
  0x00000D6C,0x00000001,0x00000018,0x00000205,0x00000000,0x00000D78,0x00000001,0x00000019,
  0x0000020B,0x00000000,0x00000D84,0x00000001,0x0000001A,0x00000211,0x00000000,0x00000D90,
  0x00000001,0x0000001B,0x00000217,0x00000000,0x00000D9C,0x00000000,0xFFFFFFFF,0x00000221,
  0x00000000,0x00000E58,0x00000000,0xFFFFFFFF,0x0000022D,0x00000000,0x00000E68,0x00000001,
  0x0000001C,0x00000236,0x00000000,0x00000E74,0x00000000,0xFFFFFFFF,0x00000244,0x00000000,
  0x00000E94,0x00000001,0x0000001D,0x0000024B,0x00000000,0x00000EA0,0x00000001,0x0000001E,
  0x00000258,0x00000000,0x00000EB0,0x00000001,0x0000001F,0x00000264,0x00000000,0x00000EBC,
  0x00000000,0xFFFFFFFF,0x00000270,0x00000000,0x00000F30,0x00000000,0xFFFFFFFF,0x00000278,
  0x00000000,0x00000F3C,0x00000000,0xFFFFFFFF,0x0000027B,0x00000000,0x00000F4C,0x00000000,
  0xFFFFFFFF,0x00000281,0x00000000,0x00000F74,0x00000000,0xFFFFFFFF,0x00000287,0x00000000,
  0x00000F84,0x00000000,0xFFFFFFFF,0x00000290,0x00000000,0x00000FF4,0x00000000,0xFFFFFFFF,
  0x0000029F,0x00000000,0x00001030,0x00000000,0xFFFFFFFF,0x000002B0,0x00000000,0x00001094,
  0x00000000,0xFFFFFFFF,0x000002C4,0x00000000,0x000010F0,0x00000000,0xFFFFFFFF,0x000002CF,
  0x00000000,0x0000116C,0x00000000,0xFFFFFFFF,
};

// UNIQUELY NAMED PARSER INITIALIZATION DATA
static APG_PARSER_INIT g_sParserInit = 
{
  0,
  (void*)g_pfnCallbacks,
  (void*)g_ulaExeData
};

void* ApgServerMsgParser::m_vpParserInit = (void*)&g_sParserInit;

ApgServerMsgParser::ApgServerMsgParser()
{
  ulong ulReturn = APG_FALSE;
  ulong ulTest;

  // get memory and error contexts
  m_vpMemCtx = NULL;
  m_vpParserCtx = NULL;
  m_vpMemCtx = vpMemCtor();
  if(!m_vpMemCtx)
  {
	  goto APG_EXIT;
  }

  // initialize the parser
  ulTest = ulParserInit(m_vpParserInit, m_vpMemCtx);
  if(!ulTest)
  {
	  goto APG_EXIT;
  }
  m_vpParserCtx = vpParserCtor(m_vpParserInit, m_vpMemCtx);
  if(!m_vpParserCtx)
  {
	  goto APG_EXIT;
  }

  // success
  ulReturn = APG_TRUE;

APG_EXIT:
  if(!ulReturn)
  {
  // clean up resources
    if(m_vpParserCtx)
{
vParserDtor(m_vpParserCtx);
 m_vpParserCtx = NULL;
}
    if(m_vpMemCtx)
	{
		vMemDtor(m_vpMemCtx);
		m_vpMemCtx = NULL;
	}
  }
}

ApgServerMsgParser::~ApgServerMsgParser()
{
  // clean up resources
  if(m_vpParserCtx)
  {
	  vParserDtor(m_vpParserCtx);
  }
  if(m_vpMemCtx)
  {
	  vMemDtor(m_vpMemCtx);
  }
}

ulong ApgServerMsgParser::ulSyntaxAnalysis(ulong ulStartRule, uchar* ucpSrc, ulong ulSrcLen, void* vpData, ulong ulTrace)
{
  ulong ulReturn = APG_FALSE;
  ulong ulTest;
  m_sSrc.ucpStr = ucpSrc;
  m_sSrc.ulLen = ulSrcLen;
  ulTest = ulParserSyntaxAnalysis(m_vpParserCtx, 0, m_sSrc.ucpStr, m_sSrc.ulLen, vpData, ulTrace);
  if(!ulTest)
  {
	  goto APG_EXIT;
  }
  ulTest = ulParserState(m_vpParserCtx, &m_sState);
  if(!ulTest)
  {
	  goto APG_EXIT;
  }
  ulTest = ulParserStats(m_vpParserCtx, &m_sStats);
  if(!ulTest)
  {
	  goto APG_EXIT;
  }

  // success
  ulReturn = APG_TRUE;

APG_EXIT:
  return ulReturn;
}

ulong ApgServerMsgParser::ulSemanticAnalysis(void* vpData)
{
  return ulParserSemanticAnalysis(m_vpParserCtx, vpData);
}

// switch template
/*
  switch(ulState)
  {
  case SYN_PRE:
    break;
  case SYN_NOMATCH:
    break;
  case SYN_EMPTY:
    break;
  case SYN_MATCH:
    break;
  case SEM_PRE:
    break;
  case SEM_POST:
    break;
  }
*/

//{{CallBack.body
int ApgServerMsgParser::ParseData(uchar * ucpSrc, ulong ulSrcLen)
{
    //init psr_msg_string - used in call back functions
    psr_msg_string = (char *) ucpSrc;
    
    //init pointer to stored data
    pStoredData = &StoredData;
    int ret_val = 1;
    
    //parse data by syntax analysis
    //first parameter for ulSyntaxAnalysis is the first ID in grammar
    ret_val = ulSyntaxAnalysis(ID_MESSAGE, ucpSrc, ulSrcLen);
    
    //ret_val = ulSemanticAnalysis(NULL);
    return (ret_val);
}


//}}CallBack.body

// call back function for rule "Time"
ulong ApgServerMsgParser::pfn_Time(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_Time
    if (ulState == SYN_MATCH) 
	{
		//vymazanie frontu videnych hracov, lepsie umiestenie som nenasiel
		pStoredData->PlayersReset();
		
		//netreba, pretoze hearCount mi urcuje, kolko budem potrebovat prvkov z pola
		pStoredData->HearReset();

#ifdef WIN32
		strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
		strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
        parse_buffer[ulLen] = '\0';
        pStoredData->SetSimTime(atof((char *) parse_buffer));
    }
//}}CallBack.pfn_Time
  return ulReturn;
}

// call back function for rule "UnumNr"
ulong ApgServerMsgParser::pfn_UnumNr(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_UnumNr
    if (ulState == SYN_MATCH) 
	{
#ifdef WIN32
		strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
		strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
        parse_buffer[ulLen] = '\0';
        pStoredData->SetPlayerNr((int) atof((char *) parse_buffer));
    }
//}}CallBack.pfn_UnumNr
  return ulReturn;
}

// call back function for rule "TeamLeft"
ulong ApgServerMsgParser::pfn_TeamLeft(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TeamLeft
    if (ulState == SYN_MATCH)
	{
#ifdef WIN32
		strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
		strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
        parse_buffer[ulLen] = '\0';
        pStoredData->SetTeamIndex(TI_LEFT);
    }
//}}CallBack.pfn_TeamLeft
  return ulReturn;
}

// call back function for rule "TeamRight"
ulong ApgServerMsgParser::pfn_TeamRight(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TeamRight
    if (ulState == SYN_MATCH) 
	{
#ifdef WIN32
		strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
		strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
        parse_buffer[ulLen] = '\0';
        pStoredData->SetTeamIndex(TI_RIGHT);
    }
//}}CallBack.pfn_TeamRight
  return ulReturn;
}

// call back function for rule "GSTime"
ulong ApgServerMsgParser::pfn_GSTime(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_GSTime
    if (ulState == SYN_MATCH)
	{
#ifdef WIN32
		strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
		strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
        parse_buffer[ulLen] = '\0';
        pStoredData->SetGameTime(atof((char *) parse_buffer));
    }
//}}CallBack.pfn_GSTime
  return ulReturn;
}

// call back function for rule "PlayMode"
ulong ApgServerMsgParser::pfn_PlayMode(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_PlayMode
    if (ulState == SYN_MATCH) 
	{
#ifdef WIN32
		strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
		strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
        parse_buffer[ulLen] = '\0';
        pStoredData->SetPlayMode(StringToPlayModeID(parse_buffer));
    }
//}}CallBack.pfn_PlayMode
  return ulReturn;
}

// call back function for rule "GYRx"
ulong ApgServerMsgParser::pfn_GYRx(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_GYRx
    if (ulState == SYN_MATCH)
	{
#ifdef WIN32
		strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
		strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
        parse_buffer[ulLen] = '\0';
        pStoredData->SetGyroRate_x(atof((char *) parse_buffer));
    }
//}}CallBack.pfn_GYRx
  return ulReturn;
}

// call back function for rule "GYRy"
ulong ApgServerMsgParser::pfn_GYRy(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_GYRy
    if (ulState == SYN_MATCH)
	{
#ifdef WIN32
		strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
		strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
        parse_buffer[ulLen] = '\0';
        pStoredData->SetGyroRate_y(atof((char *) parse_buffer));
    }
//}}CallBack.pfn_GYRy
  return ulReturn;
}

// call back function for rule "GYRz"
ulong ApgServerMsgParser::pfn_GYRz(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_GYRz
    if (ulState == SYN_MATCH)
	{
#ifdef WIN32
		strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
		strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
        parse_buffer[ulLen] = '\0';
        pStoredData->SetGyroRate_z(atof((char *) parse_buffer));
    }
//}}CallBack.pfn_GYRz
  return ulReturn;
}

// call back function for rule "ObjectName"
ulong ApgServerMsgParser::pfn_ObjectName(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_ObjectName
 if (ulState == SYN_MATCH)
	{
#ifdef WIN32
		strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
		strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
        parse_buffer[ulLen] = '\0';
        curr_obj_id = StringToObjectID(parse_buffer);
    }
//}}CallBack.pfn_ObjectName
  return ulReturn;
}

// call back function for rule "SEETeamName"
ulong ApgServerMsgParser::pfn_SEETeamName(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_SEETeamName
  if(ulState == SYN_MATCH)
  {
#ifdef WIN32
	  strncpy_s(pStoredData->players[pStoredData->GetPlayersCount()].playerTeam, 100,&psr_msg_string[ulOffset],ulLen);
#else
	  strncpy(pStoredData->players[pStoredData->GetPlayersCount()].playerTeam, &psr_msg_string[ulOffset], ulLen);
#endif	  
	pStoredData->players[pStoredData->GetPlayersCount()].playerTeam[ulLen] = '\0';
  }
//}}CallBack.pfn_SEETeamName
  return ulReturn;
}

// call back function for rule "SEEPlayerID"
ulong ApgServerMsgParser::pfn_SEEPlayerID(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_SEEPlayerID
  if(ulState == SYN_MATCH)
  {
#ifdef WIN32
	  strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
	  strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
	parse_buffer[ulLen] = '\0';
	pStoredData->SetPlayerId(atoi((char*) parse_buffer));
  }
//}}CallBack.pfn_SEEPlayerID
  return ulReturn;
}

// call back function for rule "SEEx"
ulong ApgServerMsgParser::pfn_SEEx(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_SEEx
	if (ulState == SYN_MATCH)
	{
#ifdef WIN32
		strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
		strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
		parse_buffer[ulLen] = '\0';
		if (curr_obj_id == OBJ_BALL)
			pStoredData->SetBallPosition_r(atof((char *) parse_buffer));

		else if ((curr_obj_id >= OBJ_FLAG_F1L) && (curr_obj_id <= OBJ_FLAG_G2R))
			pStoredData->SetFlagVal_x(curr_obj_id, (atof((char *) parse_buffer)));
	}
//}}CallBack.pfn_SEEx
  return ulReturn;
}

// call back function for rule "SEEy"
ulong ApgServerMsgParser::pfn_SEEy(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_SEEy
	if (ulState == SYN_MATCH)
	{
#ifdef WIN32
		strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
		strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
		parse_buffer[ulLen] = '\0';
		if (curr_obj_id == OBJ_BALL)
			pStoredData->SetBallPosition_theta(atof((char *) parse_buffer));

		else if ((curr_obj_id >= OBJ_FLAG_F1L) && (curr_obj_id <= OBJ_FLAG_G2R))
			pStoredData->SetFlagVal_y(curr_obj_id, (atof((char *) parse_buffer)));
	}
//}}CallBack.pfn_SEEy
  return ulReturn;
}

// call back function for rule "SEEz"
ulong ApgServerMsgParser::pfn_SEEz(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_SEEz
	if (ulState == SYN_MATCH)
	{
#ifdef WIN32
		strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
		strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
		parse_buffer[ulLen] = '\0';
		if (curr_obj_id == OBJ_BALL)
			pStoredData->SetBallPosition_phi(atof((char *) parse_buffer));

		else if ((curr_obj_id >= OBJ_FLAG_F1L) && (curr_obj_id <= OBJ_FLAG_G2R))
			pStoredData->SetFlagVal_z(curr_obj_id, (atof((char *) parse_buffer)));
	}
//}}CallBack.pfn_SEEz
  return ulReturn;
}

// call back function for rule "SEEPx"
ulong ApgServerMsgParser::pfn_SEEPx(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_SEEPx
  if (ulState == SYN_MATCH)
  {
#ifdef WIN32
	  strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
	  strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
	  parse_buffer[ulLen] = '\0';
	  pStoredData->SetPlayer_r(atof((char*) parse_buffer));
  }
//}}CallBack.pfn_SEEPx
  return ulReturn;
}

// call back function for rule "SEEPy"
ulong ApgServerMsgParser::pfn_SEEPy(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_SEEPy
  if (ulState == SYN_MATCH)
  {
#ifdef WIN32
	  strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
	  strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
	  parse_buffer[ulLen] = '\0';
	  pStoredData->SetPlayer_theta(atof((char*) parse_buffer));
  }
//}}CallBack.pfn_SEEPy
  return ulReturn;
}

// call back function for rule "SEEPz"
ulong ApgServerMsgParser::pfn_SEEPz(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_SEEPz
  if (ulState == SYN_MATCH)
  {
#ifdef WIN32
	  strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
	  strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
	  parse_buffer[ulLen] = '\0';
	  pStoredData->SetPlayer_phi(atof((char*) parse_buffer));
	  pStoredData->PlayerCount();
  }
//}}CallBack.pfn_SEEPz
  return ulReturn;
}

// call back function for rule "HJName"
ulong ApgServerMsgParser::pfn_HJName(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_HJName
    if (ulState == SYN_MATCH)
	{
#ifdef WIN32
		strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
		strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
        parse_buffer[ulLen] = '\0';
		
		//NAO-START
		idIter = pStoredData->mJointIDMap.find(parse_buffer);
		if (idIter == pStoredData->mJointIDMap.end())
		{
	        cerr << "(NaoBehavior) unknown joint id!" << endl;
			return 99;
		}
		//NAO-STOP
    }
//}}CallBack.pfn_HJName
  return ulReturn;
}

// call back function for rule "HJax"
ulong ApgServerMsgParser::pfn_HJax(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_HJax
    if (ulState == SYN_MATCH) 
	{
#ifdef WIN32
		strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
		strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
        parse_buffer[ulLen] = '\0';
		
		//NAO-START
		jid = (*idIter).second;
		HingeJointSense sense;
		sense.angle = atof((char *) parse_buffer);
		pStoredData->mHingeJointSenseMap[jid] = sense;
		//NAO-STOP
    }
//}}CallBack.pfn_HJax
  return ulReturn;
}

// call back function for rule "FNameLF"
ulong ApgServerMsgParser::pfn_FNameLF(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_FNameLF
    if (ulState == SYN_MATCH) 
	{
        frp_curr_foot = 0;
    }
//}}CallBack.pfn_FNameLF
  return ulReturn;
}

// call back function for rule "FNameRF"
ulong ApgServerMsgParser::pfn_FNameRF(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_FNameRF
    if (ulState == SYN_MATCH)
	{
        frp_curr_foot = 1;
    }
//}}CallBack.pfn_FNameRF
  return ulReturn;
}

// call back function for rule "FRPCx"
ulong ApgServerMsgParser::pfn_FRPCx(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_FRPCx
    if (ulState == SYN_MATCH) 
	{
#ifdef WIN32
		strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
		strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
        parse_buffer[ulLen] = '\0';
        if (frp_curr_foot)
            pStoredData->SetFrpLocCoordRF_x(atof((char *) parse_buffer));
        
        else
            pStoredData->SetFrpLocCoordLF_x(atof((char *) parse_buffer));
    }
//}}CallBack.pfn_FRPCx
  return ulReturn;
}

// call back function for rule "FRPCy"
ulong ApgServerMsgParser::pfn_FRPCy(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_FRPCy
    if (ulState == SYN_MATCH) 
	{
#ifdef WIN32
		strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
		strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
        parse_buffer[ulLen] = '\0';
        if (frp_curr_foot)
            pStoredData->SetFrpLocCoordRF_y(atof((char *) parse_buffer));
        
        else
            pStoredData->SetFrpLocCoordLF_y(atof((char *) parse_buffer));
    }
//}}CallBack.pfn_FRPCy
  return ulReturn;
}

// call back function for rule "FRPCz"
ulong ApgServerMsgParser::pfn_FRPCz(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_FRPCz
    if (ulState == SYN_MATCH) 
	{
#ifdef WIN32
		strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
		strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
        parse_buffer[ulLen] = '\0';
        if (frp_curr_foot)
            pStoredData->SetFrpLocCoordRF_z(atof((char *) parse_buffer));
        
        else
            pStoredData->SetFrpLocCoordLF_z(atof((char *) parse_buffer));
    }
//}}CallBack.pfn_FRPCz
  return ulReturn;
}

// call back function for rule "FRPFx"
ulong ApgServerMsgParser::pfn_FRPFx(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_FRPFx
    if (ulState == SYN_MATCH) 
	{
#ifdef WIN32
		strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
		strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
        parse_buffer[ulLen] = '\0';
        if (frp_curr_foot)
            pStoredData->SetFrpForceVectorRF_x(atof((char *) parse_buffer));
        
        else
            pStoredData->SetFrpForceVectorLF_x(atof((char *) parse_buffer));
    }
//}}CallBack.pfn_FRPFx
  return ulReturn;
}

// call back function for rule "FRPFy"
ulong ApgServerMsgParser::pfn_FRPFy(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_FRPFy
    if (ulState == SYN_MATCH) 
	{
#ifdef WIN32
		strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
		strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
        parse_buffer[ulLen] = '\0';
        if (frp_curr_foot)
            pStoredData->SetFrpForceVectorRF_y(atof((char *) parse_buffer));
        
        else
            pStoredData->SetFrpForceVectorLF_y(atof((char *) parse_buffer));
    }
//}}CallBack.pfn_FRPFy
  return ulReturn;
}

// call back function for rule "FRPFz"
ulong ApgServerMsgParser::pfn_FRPFz(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_FRPFz
    if (ulState == SYN_MATCH) 
	{
#ifdef WIN32
		strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
		strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
        parse_buffer[ulLen] = '\0';
        if (frp_curr_foot)
            pStoredData->SetFFrpForceVectorRF_z(atof((char *) parse_buffer));
        
        else
            pStoredData->SetFFrpForceVectorLF_z(atof((char *) parse_buffer));
    }
//}}CallBack.pfn_FRPFz
  return ulReturn;
}

// call back function for rule "HEARtime"
ulong ApgServerMsgParser::pfn_HEARtime(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_HEARtime
  if (ulState == SYN_MATCH) 
  {
#ifdef WIN32
	  strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
	  strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
	  parse_buffer[ulLen] = '\0';
	  pStoredData->SetHearTime(atof((char *) parse_buffer));
  }
//}}CallBack.pfn_HEARtime
  return ulReturn;
}

// call back function for rule "HEARxy"
ulong ApgServerMsgParser::pfn_HEARxy(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_HEARxy
  if (ulState == SYN_MATCH) 
  {
#ifdef WIN32
	  strncpy_s(parse_buffer, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
	  strncpy(parse_buffer, &psr_msg_string[ulOffset], ulLen);
#endif
	  parse_buffer[ulLen] = '\0';
	  pStoredData->SetHearXY(atof((char *) parse_buffer));
  }
//}}CallBack.pfn_HEARxy
  return ulReturn;
}

// call back function for rule "HEARself"
ulong ApgServerMsgParser::pfn_HEARself(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_HEARself
  if (ulState == SYN_MATCH) 
  {
	  pStoredData->SetHearSelf(true);
  }
//}}CallBack.pfn_HEARself
  return ulReturn;
}

// call back function for rule "HEARmessage"
ulong ApgServerMsgParser::pfn_HEARmessage(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_HEARmessage
  if (ulState == SYN_MATCH) 
  {
	  //kvoli tomu, aby som nemusel 2x kopirovat to iste (naprv do parse_buffer a potom z neho do hear...message
#ifdef WIN32
	  strncpy_s(pStoredData->hear[pStoredData->GetHearCount()].message, MAX_BUFFER_SIZE,&psr_msg_string[ulOffset],ulLen);
#else
	  strncpy(pStoredData->hear[pStoredData->GetHearCount()].message, &psr_msg_string[ulOffset], ulLen);
#endif
	  pStoredData->hear[pStoredData->GetHearCount()].message[ulLen] = '\0';
	  pStoredData->HearCount();
  }
//}}CallBack.pfn_HEARmessage
  return ulReturn;
}