/*! \file ServerMsgParserOpt.h
 <pre>
 <b>File:</b> ServerMsgParserOpt.h
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 03/10/2008
 <b>Contents:</b> class ServerMsgParserData is for holding parsed data from server
 </pre>	
 */

#ifndef ApgServerMsgParserOpt_h
#define ApgServerMsgParserOpt_h

#include "../../../config/soccertypes.h"
#include "../behaviour/worldmodel/math/CoordCartesian.h"
#include "../behaviour/worldmodel/object/Player.h"
#include "../behaviour/agentModel/Joints.h"
#include "../behaviour/agentModel/Hear.h"

#define MAX_HEAR_COUNT 22
#define MAX_SEE_PLAYERS 21

#define MAX_BUFFER_SIZE 1025

//class for holding parsed data from server
class ServerMsgParserData
{
    
private:
    
	double SimTime;							//time from Time perceptor
	double GameTime;						//time from GameState perceptor
	int  PlayMode;							//playmode from GameState perceptor - ID from soccertypes.h
	CoordCartesian GyroRateValues;			//values of x, y, z axis from GyroRate perceptor
	CoordSpherical Flags[NR_FLAGS_COUNT];	//flag[flag id], NR_FLAGS_COUNT defined in soccertypes.h
	CoordSpherical BallPosition;			//ball position in polar
	CoordCartesian FrpLocCoord_LF;			//local coordinates of the force concentration point - for left foot 
	CoordCartesian FrpLocCoord_RF;			//the same for right foot
	CoordCartesian FrpForceVector_LF;		//force vector for left foot
	CoordCartesian FrpForceVector_RF;		//force vector for right foot
	int TeamIndex;							//righ or left... defined in soccertypes.h
	int PlayerNr;							//from unum
	int hearCount;							//number of hears
	int playersCount;						//number of players

public:
	Player players[MAX_SEE_PLAYERS];		//players in the field
	HearPerceptor hear[MAX_HEAR_COUNT];		//hear perceptor
	THingeJointSenseMap mHingeJointSenseMap;
	TJointIDMap mJointIDMap;

	THingeJointSenseMap& GetmHingeJointSenseMap()
	{
		return mHingeJointSenseMap;
	}

	ServerMsgParserData() 
	{ 
		mJointIDMap =  SetupJointIDMap(mJointIDMap);
	}

	~ServerMsgParserData()
	{
	}
	void SetSimTime(double val)
	{
		SimTime = val;
	}
	double GetSimTime() 
	{
		return SimTime;
	}
	void SetGameTime(double val) 
	{
		GameTime = val;
	}
	double GetGameTime() 
	{
		return GameTime;
	}
	void SetPlayMode(int pm_id)
	{
		PlayMode = pm_id;
	}
	int GetPlayMode() 
	{
		return (PlayMode);
	}
	void SetGyroRate_x(double val)
	{
		GyroRateValues.x = val;
	}
	void SetGyroRate_y(double val)
	{
		GyroRateValues.y = val;
	}
	void SetGyroRate_z(double val) 
	{
		GyroRateValues.z = val;
	}
	CoordCartesian GetGyroRate()
	{
		return	GyroRateValues;
	}
	void SetFlagVal_x(int flag_id, double val)
	{
		Flags[flag_id].r = val;
	}
	void SetFlagVal_y(int flag_id, double val)
	{
		Flags[flag_id].theta = val;
	}
	void SetFlagVal_z(int flag_id, double val)
	{
		Flags[flag_id].phi = val;
	}
	CoordSpherical GetFlagPosition(int flag_id)
	{
		return	Flags[flag_id];
	}
	void SetBallPosition_r(double val) 
	{
		BallPosition.r = val;
	}
	void SetBallPosition_theta(double val)
	{
		BallPosition.theta = val;
	}
	void SetBallPosition_phi(double val)
	{
		BallPosition.phi = val;
	}
	CoordSpherical GetBallPosition() 
	{
		return	BallPosition;
	}

	void SetFrpLocCoordLF_x(double val) 
	{
		FrpLocCoord_LF.x = val;
	}
	void SetFrpLocCoordLF_y(double val) 
	{
		FrpLocCoord_LF.y = val;
	}
	void SetFrpLocCoordLF_z(double val) 
	{
		FrpLocCoord_LF.z = val;
	}
	void SetFrpLocCoordRF_x(double val)
	{
		FrpLocCoord_RF.x = val;
	}
	void SetFrpLocCoordRF_y(double val)
	{
		FrpLocCoord_RF.y = val;
	}
	void SetFrpLocCoordRF_z(double val)
	{
		FrpLocCoord_RF.z = val;
	}
	void SetFrpForceVectorLF_x(double val)
	{
		FrpForceVector_LF.x = val;
	}
	void SetFrpForceVectorLF_y(double val)
	{
		FrpForceVector_LF.y = val;
	}
	void SetFFrpForceVectorLF_z(double val)
	{
		FrpForceVector_LF.z = val;
	}
	void SetFrpForceVectorRF_x(double val) 
	{
		FrpForceVector_RF.x = val;
	}
	void SetFrpForceVectorRF_y(double val) 
	{
		FrpForceVector_RF.y = val;
	}
	void SetFFrpForceVectorRF_z(double val)
	{
		FrpForceVector_RF.z = val;
	}
	CoordCartesian GetFrpLocCoord_LF()
	{
		return (FrpLocCoord_LF);
	}
	CoordCartesian GetFrpLocCoord_RF()
	{
		return (FrpLocCoord_RF);
	}
	CoordCartesian GetFrpForceVector_LF() 
	{
		return (FrpForceVector_LF);
	}
	CoordCartesian GetFrpForceVector_RF()
	{
		return (FrpForceVector_RF);
	}
	void SetTeamIndex (int val) 
	{
		TeamIndex = val;
	}
	void SetPlayerNr (int val) 
	{
		PlayerNr = val;
	}
	int GetPlayerNr () 
	{
		return (PlayerNr);
	}

	Player* GetPlayers()
	{
		return players;
	}

	void SetPlayerId (int val)
	{
		players[playersCount].id = val;
	}

	void SetPlayer_r(double val)
	{
		players[playersCount].position.r = val;
	}

	void SetPlayer_theta(double val)
	{
		players[playersCount].position.theta = val;
	}

	void SetPlayer_phi(double val)
	{
		players[playersCount].position.phi = val;
	}

	void PlayersReset()
	{
		playersCount = 0;
	}

	void PlayerCount()
	{
		playersCount++;
	}

	int GetPlayersCount()
	{
		return playersCount;
	}
	
	void SetHearTime(double val)
	{
		hear[hearCount].hearTime = val;
	}

	void SetHearXY(double val)
	{
		hear[hearCount].direction = val;
	}

	void SetHearSelf(bool val)
	{
		hear[hearCount].myself = val;
	}

	HearPerceptor* GetHear()
	{
		return hear;
	}

	int GetHearCount()
	{
		return hearCount;
	}

	void HearCount()
	{
		hearCount++;
	}
	void HearReset()
	{
	//	if(hearCount != 0)
	//	{
	//		for(int i=0;i<hearCount;i++)
	//		{
	//			hear[i].direction = 0;
	//			hear[i].hearTime = 0;
	//			hear[i].myself = false;
	//			hear[i].message[0] = '\0';
	//		}
			hearCount = 0;
	//}
	}
};

#endif // ApgServerMsgParserOpt_h
